import { Component, ViewChild, OnInit, OnDestroy } from '@angular/core';
import { Router } from '@angular/router';
import { Platform, Events, AlertController } from '@ionic/angular';
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { SplashScreen } from '@ionic-native/splash-screen/ngx';

import { AuthService } from './services/auth/auth-service';
import { GlobalService } from './services/global.service';
import { environment } from '../environments/environment';
import { RoutingStateService } from '../app/services/routing-state.service';
import { TabsPage } from './pages/tabs/tabs.page';
import { SelecionaFranquiaPage } from './pages/seleciona-franquia/seleciona-franquia.page';
import { UtilsProvider } from './services/utils/utils';
import { User } from './model/user';
import { ApiProvider } from './services/api/api';
import { Franquia } from './model/franquia';
import { ConfiguracaoProvider } from './services/configuracao/configuracao';
import { OneSignal, OSNotificationPayload } from '@ionic-native/onesignal/ngx';
import { isCordovaAvailable } from './services/utils/is-cordova-available';
import { Market } from '@ionic-native/market/ngx';
import { MsgProvider } from './services/msg/msg';
import { NetworkProvider, ConnectionStatus } from './services/network/network';


declare var PagSeguroDirectPayment:any;

@Component({
  selector: 'app-root',
  templateUrl: 'app.component.html',
  providers: [GlobalService]
})

export class AppComponent implements OnInit, OnDestroy  {
  
  rootPage:any = null;
  versao: string;
  versao_servidor: string;  
  
  public appPages = [
    { title: 'Quem Somos', url: '/quem-somos', class_back: 'back-azul' },
    { title: 'Vantagens', url: '/vantagens' , class_back: 'back-verde'},
    { title: 'Dúvidas Frequentes', url: '/duvidas-frequentes', class_back: 'back-amarelo'},
    { title: 'Meus Dados', url: '/meus-dados', class_back: 'back-vermelho'},
    { title: 'Termo de Uso', url: '/termo-uso', class_back: 'back-azul'},
    { title: 'Sair', url: '/sair', class_back: 'back-verde'}
  ];
 
  constructor(private platform: Platform, 
              private statusBar: StatusBar, 
              private splashScreen: SplashScreen,
              public globalService: GlobalService, 
              private authService: AuthService,              
              public events: Events,
              private router: Router,
              routingStateService: RoutingStateService,
              private api: ApiProvider,
              private configuracao: ConfiguracaoProvider,
              private oneSignal: OneSignal,
              private market: Market,
              private msg: MsgProvider,
              private alertCtrl: AlertController,
              public networkProvider: NetworkProvider
      ) {
        routingStateService.loadRouting();        
        this.versao = environment.versao;
        this.globalService.versao = this.versao;
        events.subscribe('app:irparatabpedidos', () => {
          this.IrParaTabPedidos();
        });

        events.subscribe('app:fechar', () => {
          this.exitApp();
        });

    this.platform.ready()
      .then(() => {
        console.log("Plataforma Pronta!");
        this.networkProvider.initializeNetworkEvents();

        if (isCordovaAvailable()){
          console.log("Cordova OK. OneSignal");
          this.oneSignal.startInit(environment.oneSignalAppId, environment.sender_id);
          this.oneSignal.inFocusDisplaying(this.oneSignal.OSInFocusDisplayOption.Notification);
          this.oneSignal.handleNotificationReceived().subscribe(data => this.onPushReceived(data.payload));
          this.oneSignal.handleNotificationOpened().subscribe(data => this.onPushOpened(data.notification.payload));
          this.oneSignal.endInit();
          this.oneSignal.getIds().then((id) => {
            this.globalService.TOKEN = id.pushToken;            
            console.log("TOKEN: " + this.globalService.TOKEN);            
          });
          console.log("Cordova OK. OneSignal FIM");
        }else{
          console.log("Cordova NAO DISPONIVEL");
        }
        // Okay, so the platform is ready and our plugins are available.
        // Here you can do any higher level native things you might need.
        //statusBar.styleBlackTranslucent()
        //statusBar.styleDefault();
        //this.toast.create({ message: 'App Pronta!', position: 'botton', duration: 3000 }).present();
              
        this.statusBar.backgroundColorByHexString('#044d44');

        this.carregaJavascriptPagseguro()
        .then(
          () =>{
            console.log("carregaJavascriptPagseguro OK");
            this.globalService.setStatusPagSeguro(true);
          }
        )

        console.log("Status NET: " + this.networkProvider.status);
        //this.rootPage = CarregandoPage;    

        this.loadApp()
        .then(
          () =>{              
              this.openHomePage(splashScreen);
          } 
        )
        .catch(
          (error) => {
            this.openHomePage(splashScreen);
            alert('Erro ao carregar aplicação. Erro: ' + error.message);
          }
        )      

        }
      )
      .catch(
        (erro)=>{
          alert('Erro ao carregar plataforma: ' + erro.message);          
        }
      );

  }

  private onPushReceived(payload: OSNotificationPayload) {
    alert('Push recevied:' + payload.body);
  }
  
  private onPushOpened(payload: OSNotificationPayload) {
    alert('Push opened: ' + payload.body);
  }
  
  ngOnInit() {
    console.log("### APP ngOnInit");
  }
  
  ngOnDestroy() {
    console.log('### APP ngOnDestroy');  
  }  

  loadApp(){
    var that = this;
    return Promise.resolve() 
    .then(function() {
      console.log("loadApp ...");
      return that.testeConexao();
    })    
    .then(function() {
      console.log("testeConexao OK");
      return that.getVersaoAtual();
    })    
    .then(function() {
      console.log("loadApp 0 getVersaoAtual OK");
      return that.testeVersaoAplicativo();
    })        
    .then(function() {
      console.log("loadApp 0.1 testeVersaoAplicativo OK");
      return that.getUsuarioLocal();
    })        
    .then(function() {
      console.log("loadApp 1 getUsuarioLocal OK")            
      return that.getFranquiaLocal();        
    })     
    .then(function() {
      console.log("loadApp 2 getFranquiaLocal OK")            
      return that.getCarrrinhoLocal();        
    })      
    .then(function() {
      console.log("loadApp 3 getCarrrinhoLocal OK");           
      return that.carregaFiltroStatus();                    
    })          
    .then(function() {
         console.log("loadApp 5 carregaFiltroStatus OK")                    
    });      
  }  

  carregaFiltroStatus(){
    return new Promise((resolve, reject) => {
       this.configuracao.getFiltroStatus()
       .then(
         (status) =>{
            if (status){
              console.log("getFiltroStatus status: "+status);
              this.globalService.filtro_status = status;
            }else{
              console.log("getFiltroStatus TODOS");
              this.globalService.filtro_status = 'todos';
              this.configuracao.setFiltroStatus('todos');
            }
            resolve(true);
         }
       )
       .catch(
         (erro) =>{
           reject(erro);
         }
       )
       
    });   
  }

  getUsuarioLocal(){
    return this.authService.getUsuarioLocal()
    .then(
      (usuario:any) =>{
        if (!UtilsProvider.isEmptyObj(usuario)){
          this.api.getLogin(usuario.CPF, usuario.EMAIL)
          .then(
            (usuario: User) => {
              console.log("Dados do Usuário Atualizado com servidor!");
              this.authService.setUsuarioLogado(usuario);                   
            }
          )
          .catch(
            (error) =>{
              console.error("Erro ao atualizar dados do usuário: " + error.message);
              this.authService.setUsuarioLogado(null);              
            }
          )
        }else{
          console.log("Usuário VAZIO!");
        }  
      }
    );
  }

  getFranquiaLocal(){
    return this.globalService.getFranquiaLocal()
    .then(
      (franquia:any) =>{
        if (!UtilsProvider.isEmptyObj(franquia)){
          this.api.getFranquia(franquia.CODIGO_FRANQUIA)
          .then(
            (franquiaServer: Franquia) => {
              console.log("Dados da Franquia Atualizado com servidor!");
              this.globalService.franquia = franquiaServer;
              this.configuracao.setFranquia(franquiaServer);
              
            }
          )
        }else{
          console.log("Franquia VAZIA!");
        }  
      }
    )
    .catch(
      (error) => {
        console.error("Error: " + error.message);
      }
    );
  }

  getCarrrinhoLocal(){
    return this.globalService.getCarrinhoLocal();
  }

  getVersaoAtual(){
    return this.api.versaoAtualAplicativo()
    .then(
      (result: any) => {
        console.log("VERSAO DO SERVIDOR : "+ result.versao_atual);
        this.versao_servidor = result.versao_atual;
        
         
      }
    );
  }

  testeVersaoAplicativo(){
    return new Promise((resolve, reject) => {

      if (this.versao != this.versao_servidor){
        this.alertCtrl.create({
          header: 'Aviso',
          message: 'Existe uma nova versão do aplicativo, realize a atualização!',
          buttons: [
              {
                  text: 'ok',
                  handler: () => {
                    console.log("Abrindo Loja do Aplicativo!");
                    this.market.open('br.com.clubkidsoficial');
                    resolve(true);
                    this.exitApp();
                  }
              }
          ]
        })
        .then(
          (alert) => alert.present()           
        );        
      }else{
        resolve(false);
      }      
    });    
    
  }

  testeConexao(){
    return new Promise((resolve, reject) => {

      if (this.networkProvider.status  === ConnectionStatus.Offline){
        this.alertCtrl.create({
          header: 'Erro de conexão',
          message: 'Não foi possivel fazer conexão com o servidor. Verifique sua conexão com a internet e tente novamente!',
          buttons: [
              {
                  text: 'TENTAR NOVAMENTE',
                  handler: () => {
                    this.restartApp();
                  }
              }
          ]
        })
        .then(
          (alert) => alert.present()           
        );        
      }else{
        resolve(false);
      }      
    });    
    
  }

  private openHomePage(splashScreen: SplashScreen) {
    splashScreen.hide();
    console.log("openHomePage"); 
    if (this.globalService.franquia)
    {
      console.log("indo para page TABS");       
      this.router.navigateByUrl('/tabs');      
    }else{
      console.log("indo para page FRANQUIAS");       
      this.router.navigateByUrl('/seleciona-franquia');      
    }    
  }

  openPage(page) {
    // Reset the content nav to have just this page
    // we wouldn't want the back button to show in this scenario
     if (page.url == '/sair'){
       console.log("page.url: " +page.url);
       this.authService.signOut();      
     }else{
       console.log("page.url: " +page.url);
        this.router.navigateByUrl(page.url);
     }  
    //this.nav.push(page.component);    
  }

  IrParaTabPedidos(){
    this.router.navigateByUrl('/tabs/(pedidos:pedidos)');
    
  }

  exitApp(){
    console.log("App exitApp");
    navigator['app'].exitApp();
  } 

  restartApp(){
    console.log("App restartApp");
    window.location.reload();
  } 
  
  //CARREGA O JAVASCRIPT DO PAGSEGURO (A EXPLICAÇÃO ESTA FUNÇÃO ESTÁ LOGO ABAIXO)
  carregaJavascriptPagseguro(){
      
    if(!this.globalService.getStatusPagSeguro())
    {
      //SEJA O JAVASCRIPT NO CABEÇÁRIO DA PÁGINA
      return new Promise((resolve) => {
        let script: HTMLScriptElement = document.createElement('script');
        script.addEventListener('load', r => resolve());
        script.src = environment.script_pagseguro;
        document.head.appendChild(script);      
      });
      
    }

  }

}



