import { NgModule, ErrorHandler, LOCALE_ID, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { RouterModule, RouteReuseStrategy, Routes } from '@angular/router';

import { AppRoutingModule } from './app-routing.module';

import { registerLocaleData } from '@angular/common';
import localePt from '@angular/common/locales/pt';

import { BrowserModule } from '@angular/platform-browser';
import { IonicModule, IonicRouteStrategy } from '@ionic/angular';
import { AppComponent } from './app.component';

//import { Network } from '@ionic-native/network';
import { Camera } from '@ionic-native/camera/ngx';
import { File } from '@ionic-native/file/ngx';
import { FileTransfer } from '@ionic-native/file-transfer/ngx';
import { FilePath } from '@ionic-native/file-path/ngx';
import { SplashScreen } from '@ionic-native/splash-screen/ngx';
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { SQLite } from '@ionic-native/sqlite/ngx';

import { FormsModule, ReactiveFormsModule  } from '@angular/forms';
import { BrMaskerModule } from 'br-mask';


import { HttpClientModule } from '@angular/common/http';
import {HttpModule} from "@angular/http";

import { ConfiguracaoProvider } from './services/configuracao/configuracao';
import { CarrinhoLocalProvider } from './services/carrinho/carrinhoLocal';
import { AuthService } from './services/auth/auth-service';
import { GlobalService } from './services/global.service';
import { MsgProvider } from './services/msg/msg';

import { SearchPipe } from '../pipes/search/search';
import { SortPipe } from '../pipes/sort/sort';

import { IonicStorageModule } from '@ionic/storage';
import { PipesModule } from '../pipes/pipes.module';

import { ApiProvider } from './services/api/api';
import { UtilsProvider } from './services/utils/utils';
import { LocalStorageProvider } from './services/local-storage/local-storage';

import { Network } from '@ionic-native/network/ngx';
import { NetworkProvider } from './services/network/network';

import { FakeRequestProvider } from './services/fake-request/fake-request';

import { OneSignal } from '@ionic-native/onesignal/ngx';
import { Market } from '@ionic-native/market/ngx';
import { WebView } from '@ionic-native/ionic-webview/ngx';
import { IonicImageLoader } from 'ionic-image-loader';



// import { registerLocaleData } from '@angular/common';
// import localePtBr from '@angular/common/locales/pt';
registerLocaleData(localePt);

@NgModule({
  declarations: [
    AppComponent,
    SearchPipe,
    SortPipe
  ],
  imports: [
    BrowserModule,
    IonicModule.forRoot(),
    AppRoutingModule,
    HttpClientModule,
    HttpModule,
    IonicStorageModule.forRoot(),
    FormsModule,
    ReactiveFormsModule,    
    PipesModule,    
    BrMaskerModule,
    IonicImageLoader.forRoot()
  ],
  bootstrap: [AppComponent],
  entryComponents: [   
    AppComponent    
  ],
  providers: [
    StatusBar,
    SplashScreen,
    { provide: RouteReuseStrategy, useClass: IonicRouteStrategy },    
    //Network,
    Camera,
    File,
    FileTransfer,    
    FilePath,
    // Grande sacada para formatar numeros e datas no formato brasileiro
    {provide: LOCALE_ID, useValue: 'pt-BR'},
    //{provide: ErrorHandler, useClass: IonicErrorHandler},
    SQLite,
    CarrinhoLocalProvider,
    ConfiguracaoProvider,    
    AuthService,
    GlobalService,
    MsgProvider,
    ApiProvider,
    UtilsProvider,
    LocalStorageProvider,
    Network,
    NetworkProvider,
    FakeRequestProvider,
    OneSignal,
    Market,
    WebView
  ],
  schemas: [ CUSTOM_ELEMENTS_SCHEMA ]
})
export class AppModule {}

