import { Component, OnInit } from '@angular/core';
import { Router, NavigationEnd } from '@angular/router';
import { environment } from '../../../environments/environment';
import { GlobalService } from '../../services/global.service';
import { AuthService } from '../../services/auth/auth-service';
import { ApiProvider } from '../../services/api/api';
import { MsgProvider } from '../../services/msg/msg';
import { CarrinhoLocalProvider, ItemCarrinho } from '../../services/carrinho/carrinhoLocal';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';
import { ImageLoaderConfigService } from 'ionic-image-loader';

/**
 * Generated class for the BrinquedoDetalhePage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */
@Component({
  selector: 'app-brinquedo-detalhe',
  templateUrl: './brinquedo-detalhe.page.html',
  styleUrls: ['./brinquedo-detalhe.page.scss'],
})
export class BrinquedoDetalhePage implements OnInit, OnEnter{
  produto: any;
  url_base_fotos: any; 
  subscription: Subscription;
  htmlStrProdutoDescricao: string = '';
  
  constructor(private api: ApiProvider,
              private authService: AuthService,
              public globalService: GlobalService,              
              private carrinhoLocalProvider: CarrinhoLocalProvider,
              private msg: MsgProvider,
              private router: Router,
              private imageLoaderConfig: ImageLoaderConfigService
             ) {
                      
    this.url_base_fotos = environment.url_base_fotos;

    this.imageLoaderConfig.enableSpinner(true);
    this.imageLoaderConfig.setConcurrency(10);

    this.htmlStrProdutoDescricao = this.globalService.produto.DESCRICAO;
  
  }

  public async ngOnInit(): Promise<void> {
    await this.onEnter();

    this.subscription = this.router.events.subscribe((event:any) => {
        if (event instanceof NavigationEnd && event.url === '/brinquedo-detalhe') {
            this.onEnter();
        }
    });
  }  
  
  ionViewDidLoad() {
    console.log('ionViewDidLoad BrinquedoDetalhePage');
  }

  public async onEnter(): Promise<void> {
    this.produto = this.globalService.produto; 
    console.log('onEnter produto: ' + this.produto);
  } 

  adicionarCarinho(produto){
    let item: ItemCarrinho = new ItemCarrinho();
    item.CODIGO_PRODUTO = produto.CODIGO_PRODUTO;
    item.FOTO = produto.FOTO;
    item.QUANTIDADE = 1;
    item.VALOR = produto.VALOR;
    item.TITULO = produto.TITULO;
    item.QUANTIDADE_DISPONIVEL = produto.QUANTIDADE;
    

    this.carrinhoLocalProvider.insertItem(item);
    
  }

  aviseMe(produto){
    if (this.authService.logado()){
      this.api.produtoIndisponivel(produto, this.authService.usuarioLogado(), this.globalService.franquia.CODIGO_FRANQUIA)
      .then(
        () => this.msg.showToast('Alerta cadastrado com sucesso!')
      )
      .catch(
        (error) => this.msg.showToast('Erro ao enviar alerta!')
      );
    }else{
      this.router.navigateByUrl('/login');      
    }          
  }
}


