import { ApiProvider } from '../../services/api/api';
import { Component, OnInit, OnDestroy, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { LoadingController, IonInfiniteScroll , AlertController, ModalController, MenuController,Events, PopoverController} from '@ionic/angular';
import { environment } from '../../../environments/environment';
import { MenuStatusPage } from '../../pages/menu-status/menu-status.page';
import { CarrinhoLocalProvider, ItemCarrinho } from '../../services/carrinho/carrinhoLocal';
import { GlobalService } from '../../services/global.service';
import { AuthService } from '../../services/auth/auth-service';
import { MsgProvider } from '../../services/msg/msg';
import { UtilsProvider } from '../../services/utils/utils';
import { ConfiguracaoProvider } from '../../services/configuracao/configuracao';
import { BrinquedosPesquisaPage } from '../../pages/brinquedos-pesquisa/brinquedos-pesquisa.page';
import { ImageLoaderConfigService } from 'ionic-image-loader';
import { ImageAttribute } from 'ionic-image-loader'

/**
 * Generated class for the BrinquedosPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@Component({
  selector: 'app-brinquedos',
  templateUrl: './brinquedos.page.html',
  styleUrls: ['./brinquedos.page.scss'],
})
export class BrinquedosPage implements OnInit, OnDestroy {
  
  @ViewChild(IonInfiniteScroll ) ionInfiniteScroll : IonInfiniteScroll;

  url_base_fotos: any; 
  produtos: string[];

  isOpen: boolean = false;

  errorMessage: string;
  descending: boolean = false;
  order: number;
  column: string = 'TITULO';
  terms: string;  
  //page: number;  
  temp:Array<any>=[null,null,null,null];
  list:Array<String>=['Todos Brinquedos','Disponíveis','Indisponíveis'];

  filtro_status: string;
  loader: any;
  page: number;
  imageAttributes: ImageAttribute[] = [];

  placeholder = 'assets/imgs/default-img.jpg';

  //@ViewChild(InfiniteScroll) infiniteScroll: InfiniteScroll;

  constructor(private api: ApiProvider,
              public menu: MenuController,
              private carrinhoLocalProvider: CarrinhoLocalProvider,
              public globalService: GlobalService,
              public popoverController: PopoverController,
              private authService: AuthService,
              private alertCtrl: AlertController,
              public events: Events,
              public loadingCtrl: LoadingController,
              private router: Router,
              private msg: MsgProvider,
              private config: ConfiguracaoProvider,
              public modalCtrl: ModalController,
              private imageLoaderConfig: ImageLoaderConfigService              
              ) {
      this.url_base_fotos = environment.url_base_fotos;
      // disable spinners by default, you can add [spinner]="true" to a specific component instance later on to override this
      this.imageLoaderConfig.enableSpinner(true);
            
      // set the maximum concurrent connections to 10
      this.imageLoaderConfig.setConcurrency(10);
      //this.imageLoaderConfig.setBackgroundSize('cover');
      //this.imageLoaderConfig.enableFallbackAsPlaceholder(true);
      //this.imageLoaderConfig.setBackgroundSize('cover');

      this.imageLoaderConfig.setMaximumCacheAge(7 * 24 * 60 * 60 * 1000);// 7 days

      //this.imageLoaderConfig.setFallbackUrl('assets/imgs/default-img.jpg');
      //this.imageLoaderConfig.enableFallbackAsPlaceholder(true);
 
      
      this.imageAttributes.push({
        element: 'class',
        value: 'imagemProduto'
      })

      events.subscribe('produtos:atualizar', () => {
          this.atualizaProdutos();
      });
      events.subscribe('produtos:atualizarsemload', () => {
         this.atualizaProdutos();
      });
   }

  ngOnInit() {
    console.log("Brinquedos ngOnInit");
    if (!UtilsProvider.isEmptyObj(this.globalService.franquia)){
      console.log("Brinquedos franquia: ");
      this.atualizaProdutos();
    }else{
      console.log("Brinquedos franquia VAZIA!");
    }   
  }
  ngOnDestroy() {
    console.log('ngOnDestroy Brinquedos');      
  }  

  ionViewDidEnter() {    
    console.log('Brinquedos ionViewDidEnter');        
  }


  getAllProdutos(page: number, infiniteScroll: any) {
    return new Promise((resolve, reject) => {
      this.api.getProdutosPaginacao(
        this.globalService.franquia.CODIGO_FRANQUIA, 
        page,
        this.globalService.filtro_status,
        this.globalService.filtro_pesquisa)
        .then((result: any) => {
          for (var i = 0; i < result.data.length; i++) {
            
            var produto = result.data[i];
            this.produtos.push(produto);
          }

          if (infiniteScroll){
            infiniteScroll.target.complete();
          }
          
          // if (this.ionInfiniteScroll) {
          //   this.ionInfiniteScroll.complete();
          //   // if (this.produtos.length == result.total) {
          //   //   this.ionInfiniteScroll.disabled(false);
          //   // }
          // }
          resolve(result);
        })
        .catch((error: any) => {      
          // if (this.ionInfiniteScroll) {
          //   this.ionInfiniteScroll.complete();           
          // }  
          if (infiniteScroll){
            infiniteScroll.target.complete();
          }
          this.msg.showAlertErro('Erro ao listar os produtos. Erro: ' + error);
          reject(error);
        });
    });    
  }

  doInfinite(infiniteScrollEvent) {
    setTimeout(() => {
      this.page += 1;
      this.getAllProdutos(this.page, infiniteScrollEvent);
      //infiniteScrollEvent.target.complete();
    }, 500);
    // this.msg.showToast("Begin async operation");
    // return new Promise((resolve, reject) => {
    //   setTimeout(() => {
    //     this.page += 1;
    //     this.msg.showToast("Page: " + this.page);
    //     this.getAllProdutos(this.page)
    //     .then(
    //       () => resolve()
    //     )
    //     .catch(
    //       (erro) => {
    //         this.msg.showToast("ERRO async operation");
    //         reject(erro);
    //       }
    //     );
    //   }, 500);
    // })  
  }

  atualizaProdutos(){    
    this.produtos = [];
    this.page = 1;
    return this.getAllProdutos(this.page, null);    
  }


  saibaMais(produto){
    this.globalService.produto = produto;
    this.router.navigateByUrl('/brinquedo-detalhe');
    //const root = this.app.getRootNavs();
    //root[0].push(BrinquedoDetalhePage, {produto: produto});
  }

  adicionarCarinho(produto){
    let item: ItemCarrinho = new ItemCarrinho();
    item.CODIGO_PRODUTO = produto.CODIGO_PRODUTO;
    item.FOTO = produto.FOTO;
    item.QUANTIDADE = 1;
    item.VALOR = produto.VALOR;
    item.TITULO = produto.TITULO;
    item.QUANTIDADE_DISPONIVEL = produto.QUANTIDADE;
    

    this.carrinhoLocalProvider.insertItem(item);
    
  }

  aviseMe(produto){
    if (this.authService.logado()){
      this.api.produtoIndisponivel(produto, this.authService.usuarioLogado(), this.globalService.franquia.CODIGO_FRANQUIA)
      .then(
        () => this.msg.showToast('Alerta cadastrado com sucesso!')
      )
      .catch(
        (error) => this.msg.showToast('Erro ao enviar alerta!')
      );
    }else{
      this.router.navigateByUrl('/login');
    }      
    //this.navCtrl.push(CarrinhoPage);
  }
  

  doRefresh(refresher){
    this.atualizaProdutos()
    .then((result: any) => {
        refresher.target.complete();
      }
    )
    .catch(
      (error: any) => {
        refresher.target.complete();
      }  
    );    
  }

  async openPopover(ev: Event) {
    const popover = await this.popoverController.create({
      component: MenuStatusPage,
      event: ev      
    });
    await popover.present()
    .then(
      () => {
        console.log("THEN");
      }  
    );

    await popover.onDidDismiss().
      then((result:any)=>{
        if (result.data){
          console.log("dismiss result: "+ result.data);
          this.mudaStatus(result.data);
          
        }else{
          console.log("dismiss NULO");
        }
    });
  }

  async brinquedosPesquisa() {
    const modal = await this.modalCtrl.create({
      component: BrinquedosPesquisaPage,
      componentProps: { filtro_pesquisa: '' }
    });
    await modal.present();

    const { data } = await modal.onWillDismiss();

    console.log("Fechou Pesquisa Data: "+data);
    if (data) {
      this.setFiltroPesquisa(data);      
    }else{
      this.setFiltroPesquisa('');      
    }
  }
  

  mudaStatus(status: string){
    
    this.globalService.filtro_status = status;   
    this.config.setFiltroStatus(status);
    this.atualizaProdutos()
    .catch(
      (error: any) => {
        this.msg.showAlertErro('Erro ao listar os produtos. Erro: ' + error);
      }  
    );    
  }

  limpaFitros(){
    this.globalService.filtro_pesquisa = '';   
    this.config.setFiltroPesquisa('');

    this.globalService.filtro_status = 'todos';   
    this.config.setFiltroStatus('todos');

    this.atualizaProdutos()
    .catch(
      (error: any) => {
        this.msg.showAlertErro('Erro ao listar os produtos. Erro: ' + error);
      }  
    );        
  }
  
  setFiltroPesquisa(filtro_pesquisa: string){
    
    this.globalService.filtro_pesquisa = filtro_pesquisa;   
    this.config.setFiltroPesquisa(filtro_pesquisa);
    this.atualizaProdutos()
    .catch(
      (error: any) => {
        this.msg.showAlertErro('Erro ao listar os produtos. Erro: ' + error);
      }  
    );    
  }

  possuiFiltro(){
    return this.possuiFiltroPesquisa() ||
           this.possuiFiltroStatus();
  }

  possuiFiltroPesquisa(){
    return !UtilsProvider.empty(this.globalService.filtro_pesquisa);
  }
  
  possuiFiltroStatus(){
    return (this.globalService.filtro_status == 'disponivel') ||
           (this.globalService.filtro_status == 'indisponivel') ;
  }

  getStatus(){
    if (this.globalService.filtro_status =='disponivel'){
      return 'Disponíveis'
    }else if (this.globalService.filtro_status =='indisponivel'){
      return 'Indisponíveis'
    }    
  }

  async mudarFranquia(){
    const confirm = await this.alertCtrl.create({
      header: "Deseja realmente mudar de cidade?",
      message: "",
      buttons: [
        {
          text: "Não",
          handler: () => {
            console.log("Disagree clicked");
          }
        },
        {
          text: "Sim",
          handler: () => {
            this.router.navigateByUrl('/seleciona-franquia');
          }
        }
      ]
    });
    await confirm.present();
  }
    
}


