import { Component, ViewChild  } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import { ApiProvider } from '../../services/api/api';
import { MsgProvider } from '../../services/msg/msg';
import { Router } from '@angular/router';
import { GlobalService } from '../../services/global.service';
import { UtilsProvider } from '../../services/utils/utils';
import { AuthService } from '../../services/auth/auth-service';
import { User } from '../../model/user';

/**
 * Generated class for the CadastroUsuarioPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */
@Component({
  selector: 'app-cadastro-usuario',
  templateUrl: './cadastro-usuario.page.html',
  styleUrls: ['./cadastro-usuario.page.scss'],
})
export class CadastroUsuarioPage {
  
  form: FormGroup;
  CPFCadastrado: boolean;
  @ViewChild('inputNUMERO') inputNUMERO; 


  constructor(private formBuilder: FormBuilder,
              private api: ApiProvider,              
              private msg: MsgProvider,
              private router: Router,
              private globalService: GlobalService,
              private authService: AuthService
              ) {
    this.createForm();   
  }

  createForm() {
    this.CPFCadastrado = false;

    this.form = this.formBuilder.group({
      CODIGO_FRANQUIA: this.formBuilder.control('', [Validators.required]),
      NOME: this.formBuilder.control('', [Validators.required]),
      EMAIL: this.formBuilder.control('', [Validators.required, Validators.minLength(5)]),
      CPF: this.formBuilder.control('', [Validators.required, Validators.minLength(14)]),
      CELULAR: this.formBuilder.control('', [Validators.required]),
      CEP: this.formBuilder.control('', [Validators.required]),
      ENDERECO: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      NUMERO: this.formBuilder.control('', [Validators.required]),
      COMPLEMENTO: this.formBuilder.control(''),
      BAIRRO: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_CEP_CIDADE: this.formBuilder.control('', [Validators.required]),
      CIDADE: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_CEP_UF: this.formBuilder.control('', [Validators.required]),
      UF: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      LEITURA_TERMO: this.formBuilder.control(false, [Validators.required])
    })
    //this.form.get('COMPLEMENTO').clearValidators();
    
     
  }  

  ionViewDidLoad() {
    console.log('ionViewDidLoad CadastroUsuarioPage');
  }
  
  changeCPF(){
    this.CPFCadastrado = false;
  }

  onExitCPF(){
    if (UtilsProvider.removeMascara(this.form.controls["CPF"].value) != ""){
      this.api.existeCPF(UtilsProvider.removeMascara(this.form.controls["CPF"].value))
      .then(
        (cliente) =>{
          if (!UtilsProvider.isEmptyObj(cliente)){
            console.log("CPF CASDATRADO");
            this.CPFCadastrado = true;
          }else{
            console.log("CPF NAO CADASTRADO");
            this.CPFCadastrado = false;
          }
        }  
      );  
    }else{
      this.CPFCadastrado = false;
      console.log("CPF VAZIO");
    }
  }

  onSubmit() {
    console.log("onSubmit: this.globalService.franquia.CODIGO_FRANQUIA: "+this.globalService.franquia.CODIGO_FRANQUIA);    
    this.form.controls['CODIGO_FRANQUIA'].setValue(this.globalService.franquia.CODIGO_FRANQUIA);    
    console.log("onSubmit: ENDERECO: "+this.form.controls["ENDERECO"].value);    

    if (this.form.valid) {
      if (this.form.controls['LEITURA_TERMO'].value != true){
        this.msg.showToast('É necessário ler o termo de uso');
      }else{
        this.api.existeCPF(UtilsProvider.removeMascara(this.form.controls["CPF"].value))
        .then(
          (cliente) =>{
            if (UtilsProvider.isEmptyObj(cliente)){  
              
              let usuario: User = new User();

              //usuario.CODIGO_CLIENTE = 
              usuario.CODIGO_FRANQUIA = this.form.controls['CODIGO_FRANQUIA'].value;
              usuario.CPF = UtilsProvider.removeMascara(this.form.controls["CPF"].value);
              usuario.NOME = this.form.controls["NOME"].value;
              usuario.EMAIL  = this.form.controls["EMAIL"].value;      
              usuario.CELULAR = UtilsProvider.removeMascara(this.form.controls["CELULAR"].value);
              usuario.CEP = UtilsProvider.removeMascara(this.form.controls["CEP"].value);
              usuario.ENDERECO = this.form.controls["ENDERECO"].value;
              usuario.BAIRRO = this.form.controls["BAIRRO"].value;
              usuario.NUMERO = this.form.controls["NUMERO"].value;
              usuario.COMPLEMENTO = this.form.controls["COMPLEMENTO"].value;        
              usuario.CODIGO_CEP_CIDADE = this.form.controls["CODIGO_CEP_CIDADE"].value;        
              usuario.CODIGO_CEP_UF = this.form.controls["CODIGO_CEP_UF"].value;

              this.api.cadastrarUsuario(usuario)
              .then(
                () => {
                  //LOGIN AUTOMATICO
                  this.authService.login(UtilsProvider.removeMascara(this.form.controls["CPF"].value), this.form.controls["EMAIL"].value, this.globalService.redirect_login);                  
              })
              .catch((e) => {
                this.msg.showToast('Erro ao salvar o usuário. Erro: ' + e.message);            
                console.error(e);
              })              
            }else{
              this.msg.showAlert("CPF já cadastrado");
            }
          }  
        );
      }

    }else{
      let vlmessagem: string = 'Preencha o campo ';
      const controls = this.form.controls;
      for (const name in controls) {
        if (controls[name].invalid) {
          vlmessagem = vlmessagem + name;
          break;
        }
      }
      
      this.msg.showToast(vlmessagem);      
    }
  }  


  consultaCEP(){
    if (this.form.controls['CEP'].valid){
      this.api.consultaCEP(UtilsProvider.removeMascara(this.form.controls['CEP'].value))
      .then(
        (dados: any) =>{
          if (UtilsProvider.isEmptyObj(dados)){
            this.msg.showToast('CEP não localizado!');
          }else{
            this.form.controls['ENDERECO'].setValue(dados[0].ENDERECO);
            this.form.controls['BAIRRO'].setValue(dados[0].BAIRRO);
            this.form.controls['CODIGO_CEP_CIDADE'].setValue(dados[0].CODIGO_CEP_CIDADE);
            this.form.controls['CIDADE'].setValue(dados[0].CIDADE);
            this.form.controls['CODIGO_CEP_UF'].setValue(dados[0].CODIGO_CEP_UF);
            this.form.controls['UF'].setValue(dados[0].UF);
            this.inputNUMERO.focus();
          }
        }
      )
      .catch(
        (erro)=>{
          this.msg.showToast('Erro ao localizar cep. '+ erro.message);
        }
      );
    }else{
      this.msg.showToast('Informe um cep Válido!');
    }
  }

  lerTermo(){
    this.router.navigateByUrl('/termo-uso');    
  }

}

