import { Component, ViewChild, OnInit } from '@angular/core';
import { LoadingController, Events } from '@ionic/angular';
import { CarrinhoLocalProvider, ItemCarrinho } from '../../services/carrinho/carrinhoLocal';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { environment } from '../../../environments/environment';
import { AuthService } from '../../services/auth/auth-service';
import { MsgProvider } from '../../services/msg/msg';
import { GlobalService } from '../../services/global.service';
import { ApiProvider } from '../../services/api/api';
import { UtilsProvider } from '../../services/utils/utils';
import { NavigationEnd, Router } from '@angular/router';
import { Cupom } from '../../model/cupom';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';
import { User } from '../../model/user';


/**
 * Generated class for the CarinhoPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@Component({
  selector: 'app-carrinho',
  templateUrl: './carrinho.page.html',
  styleUrls: ['./carrinho.page.scss'],
})
export class CarrinhoPage implements OnInit, OnEnter{

  @ViewChild('meuCheck') meuCheck;
  @ViewChild('meuCupom') meuCupom;
  @ViewChild('radioGrupo') radioGrupo;
  @ViewChild('radioTransferencia') radioTransferencia;
  @ViewChild('radioPagseguro') radioPagseguro;

  
  @ViewChild('inputNUMERO') inputNUMERO; 
  @ViewChild('inputCPF') inputCPF;
  @ViewChild('inputEMAIL') inputEMAIL;

  private subscription: Subscription;

  
  htmlStrRetirarLocal: string;
  form: FormGroup;
  cliente: User;

  url_base_fotos: any; 
  
  constructor(public carrinho: CarrinhoLocalProvider,
              private msg: MsgProvider,
              private auth: AuthService,
              private formBuilder: FormBuilder,
              public api: ApiProvider,
              public loadingCtrl: LoadingController,
              public events: Events,
              private router: Router,
              private globalService: GlobalService,
              private authService: AuthService) {
    this.url_base_fotos = environment.url_base_fotos;
    this.createForm(); 
    
    this.cliente = new User();

    console.log("#### CREATE ### carrinho.teste: "+ this.carrinho.teste);
    if (UtilsProvider.isEmptyObj(this.carrinho.itens)){
      console.log("#### CREATE ###  NULLO");
      this.carrinho.itens = new Array();
    }
    console.log("#### CREATE ### carrinho.legth: "+ this.carrinho.itens.length);

    this.htmlStrRetirarLocal = this.globalService.franquia.RETIRAR_LOCAL;
  }

  public async ngOnInit(): Promise<void> {
    await this.onEnter();

    this.subscription = this.router.events.subscribe((event:any) => {
        if (event instanceof NavigationEnd && event.url === '/tabs/meu-carrinho') {
            this.onEnter();
        }
    });
  }  

  public async onEnter(): Promise<void> {
    console.log("OnEnter CarrinhoPage");
    
    if (this.carrinho.quantidadeItens > 0){
      this.preencheDadosUsuarioLogado();
      

      //Atualiza a forma de pagamento do carrinho
      this.form.controls['FORMA_PAGAMENTO'].setValue(this.carrinho.forma_pagamento);    

      if (this.carrinho.codigo_tipo_frete){
        this.form.controls['CODIGO_TIPO_FRETE'].setValue(this.carrinho.codigo_tipo_frete);    
      }else{
        //Caso so possui uma forma de envio seta ela com padrão
        if (this.globalService.franquia.STATUS_ENTREGA_CLUBKIDS == '1' &&
          this.globalService.franquia.STATUS_ENTREGA_RETIRAR_LOCAL != '1'){
            this.form.controls['CODIGO_TIPO_FRETE'].setValue('1');
        };

        if (this.globalService.franquia.STATUS_ENTREGA_CLUBKIDS != '1' &&
          this.globalService.franquia.STATUS_ENTREGA_RETIRAR_LOCAL == '1'){
            this.form.controls['CODIGO_TIPO_FRETE'].setValue('2');
        };
      }      
    }else{
      this.limpaControles();
    }  
  } 

  setForma(value){
    this.form.controls['FORMA_PAGAMENTO'].setValue(value);
  }
  preencheDadosUsuarioLogado(){
    if (this.carrinho.quantidadeItens > 0){
      //Caso esteja vazio os dados do cliente
      //preenche com o usuario logado
      if (UtilsProvider.isEmptyObj(this.carrinho.cliente)){
        if (this.auth.logado()){
          console.log("entrou 1");
          this.carrinho.gravaCliente(this.auth.usuarioLogado());
          this.clienteToForm(this.auth.usuarioLogado());
        }
      }else{
        console.log("entrou 2");
        this.clienteToForm(this.carrinho.cliente);
      }

      
    }
  }
  
  loginAutomatico(){
    if (this.inputCPF.value && this.inputEMAIL.value){
      console.log('LoginAutomatico...');
      this.authService.login(UtilsProvider.removeMascara(this.form.controls["CPF"].value), this.form.controls["EMAIL"].value)
      .then(() => {
        console.log('Login OK!');
        this.preencheDadosUsuarioLogado();
      })
      .catch((error: any) => {
        console.log('Usuario NAO POSSUI LOGIN!');        
      });      
    }
  }
  
  selectRadio(value){
    this.carrinho.gravaFormaPagamentoCarrinho(value);
  }

  selectCodigoTipoFrete(value){
    this.carrinho.gravaCodigoTipoFrete(value);
  }

  consultaCEP(){
    if (this.form.controls['CEP'].valid){
      this.api.consultaCEP(UtilsProvider.removeMascara(this.form.controls['CEP'].value))
      .then(
        (dados: any) =>{
          if (UtilsProvider.isEmptyObj(dados)){
            this.msg.showToast('CEP não localizado!');
          }else{
            this.form.controls['ENDERECO'].setValue(dados[0].ENDERECO);
            this.form.controls['BAIRRO'].setValue(dados[0].BAIRRO);
            this.form.controls['CODIGO_CEP_CIDADE'].setValue(dados[0].CODIGO_CEP_CIDADE);
            this.form.controls['CIDADE'].setValue(dados[0].CIDADE);
            this.form.controls['CODIGO_CEP_UF'].setValue(dados[0].CODIGO_CEP_UF);
            this.form.controls['UF'].setValue(dados[0].UF);  
            this.formToCliente(this.cliente);
            this.carrinho.gravaCliente(this.cliente);
            this.inputNUMERO.focus();
          }
        }
      )
      .catch(
        (erro)=>{
          this.msg.showToast('Erro ao localizar cep. '+ erro.message);
        }
      );
    }else{
      this.msg.showToast('Informe um cep Válido!');
    }
  }

  clienteToForm(cliente: User){
    console.log("atualizaMeusPedidos");
    if (cliente){
      this.form.controls["CODIGO_FRANQUIA"].setValue(this.globalService.franquia.CODIGO_FRANQUIA);      
      this.form.controls["CODIGO_CLIENTE"].setValue(cliente.CODIGO_CLIENTE);
      this.form.controls["NOME"].setValue(cliente.NOME);
      this.form.controls["EMAIL"].setValue(cliente.EMAIL);
      this.form.controls["CPF"].setValue(UtilsProvider.mascaraCPF(cliente.CPF));
      this.form.controls["CELULAR"].setValue(UtilsProvider.mascaraCelular(cliente.CELULAR));
      this.form.controls["CEP"].setValue(UtilsProvider.mascaraCEP(cliente.CEP));
      this.form.controls["ENDERECO"].setValue(cliente.ENDERECO);
      this.form.controls["NUMERO"].setValue(cliente.NUMERO);
      this.form.controls["COMPLEMENTO"].setValue(cliente.COMPLEMENTO);
      this.form.controls["BAIRRO"].setValue(cliente.BAIRRO);
      this.form.controls["CIDADE"].setValue(cliente.CIDADE);
      this.form.controls["UF"].setValue(cliente.UF);
      this.form.controls["CODIGO_CEP_CIDADE"].setValue(cliente.CODIGO_CEP_CIDADE);
      this.form.controls["CODIGO_CEP_UF"].setValue(cliente.CODIGO_CEP_UF);
    }else{
        this.limpaControles();
    }   
  }

  formToCliente(cliente: User){
    cliente.CODIGO_CLIENTE = this.form.controls['CODIGO_CLIENTE'].value;
    cliente.CODIGO_FRANQUIA = this.form.controls['CODIGO_FRANQUIA'].value;
    cliente.CPF = UtilsProvider.removeMascara(this.form.controls["CPF"].value);
    cliente.NOME = this.form.controls["NOME"].value;
    cliente.EMAIL  = this.form.controls["EMAIL"].value;      
    cliente.CELULAR = UtilsProvider.removeMascara(this.form.controls["CELULAR"].value);
    cliente.CEP = UtilsProvider.removeMascara(this.form.controls["CEP"].value);
    cliente.ENDERECO = this.form.controls["ENDERECO"].value;
    cliente.BAIRRO = this.form.controls["BAIRRO"].value;
    cliente.NUMERO = this.form.controls["NUMERO"].value;
    cliente.COMPLEMENTO = this.form.controls["COMPLEMENTO"].value;        
    cliente.CIDADE = this.form.controls["CIDADE"].value;        
    cliente.CODIGO_CEP_CIDADE = this.form.controls["CODIGO_CEP_CIDADE"].value;        
    cliente.UF = this.form.controls["UF"].value;
    cliente.CODIGO_CEP_UF = this.form.controls["CODIGO_CEP_UF"].value;        
  }

  ionViewDidLoad() {
    console.log('ionViewDidLoad CarinhoPage');
    // if (!UtilsProvider.isEmptyObj(this.carrinho.cupom)){
    //   this.meuCupom.value = this.carrinho.cupom.CUPOM;
    // }
  }

  createForm() {

    this.form = this.formBuilder.group({
      CODIGO_FRANQUIA: this.formBuilder.control('', [Validators.required]),
      CODIGO_CLIENTE: this.formBuilder.control(''),      
      FORMA_PAGAMENTO: this.formBuilder.control('', [Validators.required]),
      NOME: this.formBuilder.control('', [Validators.required]),
      EMAIL: this.formBuilder.control('', [Validators.required, Validators.minLength(5)]),
      CPF: this.formBuilder.control('', [Validators.required, Validators.minLength(11)]),
      CELULAR: this.formBuilder.control('', [Validators.required]),
      CEP: this.formBuilder.control('', [Validators.required]),
      ENDERECO: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      NUMERO: this.formBuilder.control('', [Validators.required]),
      COMPLEMENTO: this.formBuilder.control(''),
      BAIRRO: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_CEP_CIDADE: this.formBuilder.control('', [Validators.required]),
      CIDADE: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_CEP_UF: this.formBuilder.control('', [Validators.required]),
      UF: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_TIPO_FRETE: this.formBuilder.control('', [Validators.required]),
    })

    this.form.controls["CODIGO_FRANQUIA"].setValue(this.globalService.franquia.CODIGO_FRANQUIA);
     
  }
  
  validarCupom(){
    if (this.meuCupom.value != ""){
      this.api.getCupom(this.globalService.franquia.CODIGO_FRANQUIA,
                        this.meuCupom.value)
      .then(
        (dados:any) => {
          if (dados.codigo_cupom != ""){
            let cupom = new Cupom();
            cupom = dados.cupom[0];
            console.log("Dados.CODIGO_CUPOM: " + cupom.CODIGO_CUPOM);
            console.log("Dados.CODIGO_TIPO_DESCONTO: " + cupom.CODIGO_TIPO_DESCONTO);
            console.log("Dados.CUPOM: " + cupom.CUPOM);
            console.log("Dados.VALOR: " + cupom.VALOR);
            cupom.PRODUTOS = dados.produtos;
            this.carrinho.gravaCupomCarrinho(cupom);
            
          }else{
            this.msg.showAlert("Cupom Inválido!")
          }
          
        }
      )
    }else{
      this.msg.showAlert("Informe o cupom!");
    }
   
  }


  

  adicionarItem(itemCarrinho: ItemCarrinho){
    let quantidade = itemCarrinho.QUANTIDADE + 1;
    if (quantidade <= itemCarrinho.QUANTIDADE_DISPONIVEL){
      itemCarrinho.QUANTIDADE++;      
      this.carrinho.updateItem(itemCarrinho);      
    }else{
      this.msg.showAlert("Quantidade Indisponível!")
    }
    
  }

  removerItem(itemCarrinho: ItemCarrinho) {
    if (itemCarrinho.QUANTIDADE > 1){
      itemCarrinho.QUANTIDADE--;
      
      this.carrinho.updateItem(itemCarrinho);

    }else{
      console.log("REMOVE ITEM");
      this.carrinho.removeItem(itemCarrinho)
       .then(
          () => {
            if (this.carrinho.itens.length == 0){
              //Limpa os controles
              console.log("LIMPA CONTROLES");
              this.limpaControles();
            }else{
              console.log("NAO LIMPA CONTROLES ITENS: "+this.carrinho.itens.length);
            }
          }
       );
    }  
  }  
  
  limpaControles(){
    console.log("Limpando Controles")
    this.form.controls["CODIGO_FRANQUIA"].setValue('');
    this.form.controls["CODIGO_CLIENTE"].setValue('');
    this.form.controls["FORMA_PAGAMENTO"].setValue('');
    this.form.controls["CODIGO_TIPO_FRETE"].setValue('');
    this.form.controls["NOME"].setValue('');
    this.form.controls["EMAIL"].setValue('');
    this.form.controls["CPF"].setValue('');
    this.form.controls["CELULAR"].setValue('');
    this.form.controls["CEP"].setValue('');
    this.form.controls["ENDERECO"].setValue('');
    this.form.controls["NUMERO"].setValue('');
    this.form.controls["COMPLEMENTO"].setValue('');
    this.form.controls["BAIRRO"].setValue('');
    this.form.controls["CIDADE"].setValue('');
    this.form.controls["UF"].setValue('');
    this.form.controls["CODIGO_CEP_CIDADE"].setValue('');
    this.form.controls["CODIGO_CEP_UF"].setValue('');
  }

  irParaBrinquedos(){
    this.router.navigateByUrl('/tabs/brinquedos');   
    // var t: Tabs = this.navCtrl.parent;
    // t.select(0);
  }

  alugar(){
        
    if (this.carrinho.getCarrinhoItensCount() == 0){
      this.msg.showAlert('Escolha um brinquedo para o Carrinho!');
    }else if (UtilsProvider.empty(this.radioGrupo.value)){
      this.msg.showAlert('Selecione a forma de pagamento!');
    }else if (!this.form.valid) {
      //Valida o preenchimento dos campos
      let vlmessagem: string = 'Preencha o campo ';
      const controls = this.form.controls;
      for (const name in controls) {
        if (controls[name].invalid) {
          if (name == 'CODIGO_TIPO_FRETE'){
            vlmessagem = 'Prenncha a Forma de Envio';
          }else{
            vlmessagem = vlmessagem + name;
          }
          
          break;
        }
      }
      
      this.msg.showToast(vlmessagem);
    }else if (!this.auth.logado()){
      //Caso não esteja logado
      this.cadastrarUsuario()
      .then(
        (result:boolean) =>{
          if (result){
            console.log("Usuario Cadastrado e logado com sucesso");
            this.finalizaCompra();
          }else{
            this.router.navigateByUrl('/login');
          }
        }
      )                      
    }else{
      this.finalizaCompra();
    }  
  }

  finalizaCompra(){
    console.log("Gerando Pedido...");
    console.log("CODIGO_FRANQUIA:" + this.globalService.franquia.CODIGO_FRANQUIA);
    console.log("CODIGO_CLIENTE: " + this.auth.usuarioLogado().CODIGO_CLIENTE);
      
    this.formToCliente(this.cliente);
    console.log("ENDERECO: " + this.cliente.ENDERECO);
    this.carrinho.gravaCliente(this.cliente);
      

    ///Atualiza o globalService
    this.globalService.pedido =  this.carrinho.pedido;
    this.globalService.codigo_cliente =  this.carrinho.cliente.CODIGO_CLIENTE;    
    this.globalService.valor_produto = this.carrinho.valor_produto;
    this.globalService.desconto = this.carrinho.desconto;
    this.globalService.valor_total = this.carrinho.valor_total;
    this.globalService.itens_carinho = this.carrinho.itens;
    console.log("PEDIDO GLOBAL: "+this.globalService.pedido);
    console.log("CLIENTE GLOBAL: "+this.globalService.codigo_cliente);

    //Caso já exista pedidos
    if (!UtilsProvider.empty(this.carrinho.pedido)){
      console.log("proximaPagina");
      this.proximaPagina();
    }else{
      console.log("gerarPedido");
      this.gerarPedido();
    }
  }


  cadastrarUsuario(){
    return new Promise((resolve, reject) => {
      this.api.existeCPF(UtilsProvider.removeMascara(this.form.controls["CPF"].value))
          .then(
            (cliente) =>{
              if (UtilsProvider.isEmptyObj(cliente)){  
                
                let usuario: User = new User();

                //usuario.CODIGO_CLIENTE = 
                usuario.CODIGO_FRANQUIA = this.form.controls['CODIGO_FRANQUIA'].value;
                usuario.CPF = UtilsProvider.removeMascara(this.form.controls["CPF"].value);
                usuario.NOME = this.form.controls["NOME"].value;
                usuario.EMAIL  = this.form.controls["EMAIL"].value;      
                usuario.CELULAR = UtilsProvider.removeMascara(this.form.controls["CELULAR"].value);
                usuario.CEP = UtilsProvider.removeMascara(this.form.controls["CEP"].value);
                usuario.ENDERECO = this.form.controls["ENDERECO"].value;
                usuario.BAIRRO = this.form.controls["BAIRRO"].value;
                usuario.NUMERO = this.form.controls["NUMERO"].value;
                usuario.COMPLEMENTO = this.form.controls["COMPLEMENTO"].value;        
                usuario.CODIGO_CEP_CIDADE = this.form.controls["CODIGO_CEP_CIDADE"].value;        
                usuario.CODIGO_CEP_UF = this.form.controls["CODIGO_CEP_UF"].value;

                this.api.cadastrarUsuario(usuario)
                .then(
                  () => {
                    //LOGIN AUTOMATICO
                    this.authService.login(UtilsProvider.removeMascara(this.form.controls["CPF"].value), this.form.controls["EMAIL"].value)
                    .then(
                      () =>{
                        resolve(true);           
                      }
                    )
                    .catch((e) => {
                      reject(e);
                      console.error('Erro ao realizar login automático. Erro: ' + e.message);                              
                    })                   
                })
                .catch((e) => {
                  reject(e);
                  console.error('Erro ao cadastrar Usuario. Erro: ' + e.message);                              
                })              
              }else{
                resolve(false);
              }
            }  
          );
      });        
  }

  atualizarPedidos(){
    this.events.publish('pedidos:atualizarsemload');
  }

  atualizarProdutos(){
    this.events.publish('produtos:atualizarsemload');
  }

  proximaPagina(){
    if (this.radioGrupo.value == "transferencia"){
      this.router.navigateByUrl('/transferencia');   
    }else if (this.radioGrupo.value == "pagseguro"){
      this.router.navigateByUrl('/pagseguro');   
    }
  }

  async revalidarEstoque(){
    let loader = await this.loadingCtrl.create({
      message: "Revalidando Estoque..."
    });
    loader.present();
    
    let dados = {
      CODIGO_FRANQUIA: this.globalService.franquia.CODIGO_FRANQUIA,
      ITENS: this.carrinho.itens
    }

    return this.api.revalidarCarrinho(dados)   
    .then(
       (result: any) =>{ 
         loader.dismiss();
         
         let itens = result.ITENS_SEM_ESTOQUE;

         for (var i = 0; i < itens.length; i++) {
           let item = new ItemCarrinho();
           item.CODIGO_PRODUTO = itens[i];
           this.carrinho.removeItem(item);           
         }
         this.msg.showAlert('Desculpe, durante o processo de finalização do seu pedido um ou mais brinquedos ficaram indisponíveis em nosso estoque. O carrinho será atualizado!');          
       } 
    )
    .catch(
      (error) =>{
        loader.dismiss();
        if(error.status === 422){
          this.msg.showAlert('Erro no Servidor: '+ error.error.erro_servidor);                  
        }else{
          this.msg.showAlert('Falha na comunicação com o servidor. Por favor, tente mais tarde.');
        } 
      }
    );   
    

  }

  async gerarPedido(){

    var codigo_forma_pagamento: string;

    if (this.radioGrupo.value == "pagseguro"){
      codigo_forma_pagamento = '1';
    }else if (this.radioGrupo.value == "transferencia"){
      codigo_forma_pagamento = '2';  
    } 


    let dados = {
      CODIGO_FRANQUIA: this.globalService.franquia.CODIGO_FRANQUIA,
      CODIGO_CLIENTE: this.auth.userId(),
      CODIGO_STATUS_CARRINHO: "1",
      CODIGO_FORMA_PAGAMENTO: codigo_forma_pagamento,
      CODIGO_TIPO_FRETE: this.form.controls['CODIGO_TIPO_FRETE'].value,

      ITENS: this.carrinho.itens,
      VALOR_PRODUTO: this.carrinho.valor_produto,
      DESCONTO: this.carrinho.desconto,
      VALOR_TOTAL: this.carrinho.valor_total,
      CLIENTE: this.carrinho.cliente
    }

      let loader = await this.loadingCtrl.create({
        message: "Gerando Pedido..."
      });
      loader.present();
      

      return this.api.salvaCarrinho(dados)
      .then((result: any) =>{ 
          this.globalService.pedido =  result.CODIGO_PEDIDO;
          this.carrinho.gravaPedidoCarrinho(result.CODIGO_PEDIDO)
          .then(
            (result) => {              
              loader.dismiss();
              console.log("STATUS: "+result.STATUS);
              console.log("result: "+result);

              this.proximaPagina();                  
              //Atualiza a tela de pedidos
              this.atualizarPedidos();
              //Atualiza os produtos
              this.atualizarProdutos();                
            }                            
          );       
      })
      .catch((error: any) => {
        
        loader.dismiss();

        if(error.status === 422){
          this.msg.showAlert('Erro no Servidor: '+ error.error.erro_servidor);          
        }else if(error.status === 423){
          this.revalidarEstoque();
        }else{
          this.msg.showAlert('Falha na comunicação com o servidor. Por favor, tente mais tarde.');
        } 

        
        
      });     
  
  }

  limparCarrinho(){
    this.carrinho.limpaCarrinho()
    .then(
      () =>{
        this.limpaControles();
      }
    )
  }


}


