
import { Component, OnInit, NgZone } from '@angular/core';
import { ActionSheetController,Platform, AlertController, LoadingController } from '@ionic/angular';
import { Camera, CameraOptions } from "@ionic-native/camera/ngx";
import { FileTransfer, FileUploadOptions, FileTransferObject } from '@ionic-native/file-transfer/ngx';
import { File } from '@ionic-native/file/ngx';
import { FilePath } from '@ionic-native/file-path/ngx';
import { ApiProvider } from '../../services/api/api';
import { environment } from '../../../environments/environment';
import { MsgProvider } from '../../services/msg/msg';
import { GlobalService } from '../../services/global.service';
import { NavigationEnd, Router } from '@angular/router';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';
import { UtilsProvider } from '../../services/utils/utils';

/**
 * Generated class for the DetalhePedidoPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */
declare var cordova: any;

@Component({
  selector: 'app-detalhe-pedido',
  templateUrl: './detalhe-pedido.page.html',
  styleUrls: ['./detalhe-pedido.page.scss'],
})
export class DetalhePedidoPage implements OnInit, OnEnter {
  pedido: any;
  itens: any[];
  historicos: any[];
  franquia: any;
  lastImage: string = null;
  imageURI:any;
  url_base_comprovantes: any;
  url_img_bancos: any;
  comprovante: any;

  public photos: any;
  public base64Image: string;
  public fileImage: string;
  public responseData: any;
  
  subscription: Subscription;

  constructor(private camera: Camera,
              private alertCtrl: AlertController,
              private api: ApiProvider,                            
              private transfer: FileTransfer, 
              private file: File, 
              private filePath: FilePath, 
              public actionSheetCtrl: ActionSheetController,
              public platform: Platform,
              public loadingCtrl: LoadingController,
              private msg: MsgProvider,
              private globalService: GlobalService,
              private router: Router,
              private zone: NgZone) {
  
     
  }


  mostraCPF(texto: string){
    if (texto.length == 11){
      return true;
    }else{
      return false;
    }    
  }

  mostraCNPJ(texto: string){
    if (texto.length == 14){
      return true;
    }else{
      return false;
    }    
  }

  mostraOperacao(texto: string){
    if (!UtilsProvider.empty(texto)){
      return true;
    }else{
      return false;
    }    
  }

  formataCPF(texto: string){
    return UtilsProvider.mascaraCPF(texto);
  }

  formataCNPJ(texto: string){
    return UtilsProvider.mascaraCNPJ(texto);
  }

  public async ngOnInit(): Promise<void> {
    await this.onEnter();

    this.subscription = this.router.events.subscribe((event:any) => {
        if (event instanceof NavigationEnd && event.url === '/detalhe-pedido') {
            this.onEnter();
        }
    });
  }  

  public async onEnter(): Promise<void> {
    console.log("OnEnter DetalhePedidoPage");
    
    this.pedido = this.globalService.detalhe_pedido_pedido;    
    this.itens = this.globalService.detalhe_pedido_itens;   
    this.historicos = this.globalService.detalhe_pedido_historicos

    this.zone.run(() => {
      this.comprovante = this.globalService.detalhe_pedido_comprovante;
    });

    
    this.franquia =this.globalService.franquia;

    console.log("url_base_comprovantes" +  environment.url_base_comprovantes)  
    this.url_base_comprovantes = environment.url_base_comprovantes;   
    this.url_img_bancos = environment.url_img_bancos;       
    
  } 

  ionViewDidLoad() {
    console.log('ionViewDidLoad DetalhePedidoPage');
  }

  async apagarComprovante(index) {
    let confirm = await this.alertCtrl.create({
      header: "Deseja Realmente Excluir o comprovante?!",
      message: "",
      buttons: [
        {
          text: "Não",
          handler: () => {
            console.log("Disagree clicked");
          }
        },
        {
          text: "Sim",
          handler: () => {
            console.log("Agree clicked");
            this.photos.splice(index, 1);
          }
        }
      ]
    });
    confirm.present();
  }

  existeComprovante(){
    if (this.comprovante){
      return true;
    }else{
      return false;
    }
  }
  capturarFoto() {
    console.log("Entrou capturarFoto");

    const options: CameraOptions = {
      quality: 50,
      destinationType: this.camera.DestinationType.DATA_URL,
      encodingType: this.camera.EncodingType.JPEG,
      mediaType: this.camera.MediaType.PICTURE,
      targetWidth: 450,
      targetHeight: 450,
      saveToPhotoAlbum: false
    };
    this.photos = [];

    this.camera.getPicture(options).then(
      imageData => {
        this.enviaFoto(imageData)
        .then(
          () => {
            this.base64Image = "data:image/jpeg;base64," + imageData;
            this.photos.push(this.base64Image);
            this.photos.reverse();
          }
        )
        .catch(
          (err) => {
            this.msg.showToast('Erro ao enviar foto para o servidor!');            
          }  
        );

        
      },
      err => {
        this.msg.showToast('Erro ao Capturar Foto!');
      }
    );
  }  

  enviaFoto(imageData) {
    let dados = {
      CODIGO_PEDIDO: this.pedido.CODIGO_PEDIDO,
      IMAGEM_B64: imageData
    }
    console.log("enviaFoto: " + dados);
    return this.api.anexarComprovante(dados).then(
      (result: any) => {
        console.log("FOTO ENVIADA OK! ID:" + result.ID);
        this.responseData = result;
      },
      err => {
        console.log("ERROR: " + err);
        return Promise.reject(err);        
      }
    );
  }
  
  openGallery() {
    const options: CameraOptions = {
      quality: 50,
      destinationType: this.camera.DestinationType.DATA_URL,
      encodingType: this.camera.EncodingType.JPEG,
      mediaType: this.camera.MediaType.PICTURE,
      sourceType: this.camera.PictureSourceType.SAVEDPHOTOALBUM,
      targetWidth: 450,
      targetHeight: 450
    }

    this.camera.getPicture(options).then(
      (imageData) => {
        this.enviaFoto(imageData)
        .then(
          () => {
            this.base64Image = "data:image/jpeg;base64," + imageData;
            this.photos.push(this.base64Image);
            this.photos.reverse();
          }
        )
        .catch(
          (err) => {
            this.msg.showToast('Erro ao enviar foto para o servidor!');
          }  
        );
    }, (err) => {
      this.msg.showToast('Erro ao abrir Galeria!');
  })}  


  public takePicture(sourceType) {
    // Create options for the Camera Dialog
    var options = {
      quality: 100,
      sourceType: sourceType,
      saveToPhotoAlbum: false,
      correctOrientation: true,
      destinationType: this.camera.DestinationType.FILE_URI
    };
   
    // Get the data of an image
    this.camera.getPicture(options).then((imagePath) => {
      this.imageURI = imagePath;
      // Special handling for Android library
      if (this.platform.is('android') && sourceType === this.camera.PictureSourceType.PHOTOLIBRARY) {
        this.filePath.resolveNativePath(imagePath)
          .then(filePath => {
            let correctPath = filePath.substr(0, filePath.lastIndexOf('/') + 1);
            let currentName = imagePath.substring(imagePath.lastIndexOf('/') + 1, imagePath.lastIndexOf('?'));
            this.copyFileToLocalDir(correctPath, currentName, this.createFileName(), 
              this.pedido.CODIGO_PEDIDO,
              this.globalService.franquia.CODIGO_FRANQUIA,
              this.globalService.codigo_cliente);
          });
      }else if (this.platform.is('ios') && sourceType === this.camera.PictureSourceType.PHOTOLIBRARY) {
        this.filePath.resolveNativePath(imagePath)
          .then(filePath => {
            let correctPath = filePath.substr(0, filePath.lastIndexOf('/') + 1);
            let currentName = imagePath.substring(imagePath.lastIndexOf('/') + 1, imagePath.lastIndexOf('?'));
            this.copyFileToLocalDir(correctPath, currentName, this.createFileName(), 
              this.pedido.CODIGO_PEDIDO,
              this.globalService.franquia.CODIGO_FRANQUIA,
              this.globalService.codigo_cliente);
          });
      }
       else {
        var currentName = imagePath.substr(imagePath.lastIndexOf('/') + 1);
        var correctPath = imagePath.substr(0, imagePath.lastIndexOf('/') + 1);
        this.copyFileToLocalDir(correctPath, currentName, this.createFileName(), 
          this.pedido.CODIGO_PEDIDO,
          this.globalService.franquia.CODIGO_FRANQUIA,
          this.globalService.codigo_cliente);
      }
    }, (err) => {
      this.msg.showToast('Error ao selecionar imagem');
    });
  }

  async presentActionSheet() {
    let actionSheet = await this.actionSheetCtrl.create({
      header: 'Anexar Comprovante',
      buttons: [
        {
          text: 'Usar a Galeria',
          handler: () => {
            this.takePicture(this.camera.PictureSourceType.PHOTOLIBRARY);
          }
        },
        {
          text: 'Tirar Foto',
          handler: () => {
            this.takePicture(this.camera.PictureSourceType.CAMERA);
          }
        },
        {
          text: 'Cancelar',
          role: 'cancel'
        }
      ]
    });
    actionSheet.present();
  }

  // Create a new name for the image
private createFileName() {
  var d = new Date(),
  n = d.getTime(),
  newFileName =  n + ".jpg";
  return newFileName;
}
 
// Copy the image to a local folder
async copyFileToLocalDir(namePath, currentName, newFileName, codigo_pedido, codigo_franquia, codigo_cliente) {
  
  let loader = await this.loadingCtrl.create({
    message: "Aguarde Um momento..."
  });
  await loader.present();

  await this.file.copyFile(namePath, currentName, cordova.file.dataDirectory, newFileName).then(
    success => {
    this.lastImage = cordova.file.dataDirectory + newFileName;

    let options: FileUploadOptions = {
      fileKey: "anexo",
      fileName: newFileName,
      chunkedMode: false,
      mimeType: "multipart/form-data",
      params : {'fileName': newFileName, 'CODIGO_PEDIDO': codigo_pedido, 'CODIGO_FRANQUIA': codigo_franquia, 'CODIGO_CLIENTE:':codigo_cliente}
      
  }

  //this.msg.showAlert('codigo_pedido: '+ codigo_pedido+ ' codigo_cliente:' + codigo_cliente);

  const fileTransfer: FileTransferObject = this.transfer.create();
  
  let url_comprovante = environment.api + 'anexar-comprovante';
    
  fileTransfer.upload(this.lastImage,url_comprovante,options)
    .then((data:any) => {
      // success
      loader.dismiss();
      
      var resultComprovante =JSON.parse(data.response);
      
      //Força a atualização da Variavel comprovante
      this.zone.run(() => {
        this.comprovante = resultComprovante.comprovante;
      });
      
      console.log("depois data.response:  " + this.comprovante);

      this.msg.showToast('Comprovante anexado com sucesso!');
    }, (error) => {
      // error
      loader.dismiss();
      //this.msg.showAlert('Erro no Servidor: '+ error.code);                  
      if(error.http_status === 422){
        var body =JSON.parse(error.body);
        this.msg.showAlert('Erro no Servidor: '+ body.erro_servidor);                  
      }else{
        this.msg.showAlert('Falha na comunicação com o servidor. Por favor, tente mais tarde.Codigo Erro:'+error.http_status);
      } 
      //this.msg.showAlert('Erro ao anexar comprovante.1: '+error.message);
  })

  }, error => {
    loader.dismiss();
    this.msg.showAlert('Erro ao anexar comprovante. '+error.message);
  })
  .catch(
    (error) =>{
      loader.dismiss();
      this.msg.showAlert('Erro ao anexar comprovante. '+error.message);
    }
  );
}
 

// Always get the accurate path to your apps folder
public pathForImage(img) {
  if (img === null) {
    return '';
  } else {
    return cordova.file.dataDirectory + img;
  }
}

}
