import { Component, ViewChild, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';

import { AuthService } from '../../services/auth/auth-service';
import { MsgProvider } from '../../services/msg/msg';
import { UtilsProvider } from '../../services/utils/utils';

import { NavigationEnd, Router } from '@angular/router';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';
import { GlobalService } from '../../services/global.service';
import { RoutingStateService } from '../../services/routing-state.service';

//import { HomePage } from '../home/home';
//import { SignupPage } from '../signup/signup';
//import { ResetpasswordPage } from '../resetpassword/resetpassword';

@Component({
  selector: 'app-login',
  templateUrl: './login.page.html',
  styleUrls: ['./login.page.scss'],
})
export class LoginPage implements OnInit, OnEnter{
  
  form: FormGroup;
  private subscription: Subscription;

  constructor(
    private authService: AuthService,
    private router: Router,
    private msg: MsgProvider,
    private formBuilder: FormBuilder,
    private globalService: GlobalService,
    private routingStateService: RoutingStateService) {
      this.createForm();  
  }

  createForm() {  
    this.form = this.formBuilder.group({
      CPF: this.formBuilder.control('', [Validators.required, Validators.minLength(14)]),      
      EMAIL: this.formBuilder.control('', [Validators.required, Validators.minLength(5)])
    });     
  }  

  public async ngOnInit(): Promise<void> {
    await this.onEnter();

    this.subscription = this.router.events.subscribe((event:any) => {
        if (event instanceof NavigationEnd && event.url === '/login') {
            this.onEnter();
        }
    });
  }  

  public async onEnter(): Promise<void> {
    console.log("OnEnter LoginPage");
    this.globalService.redirect_login = this.routingStateService.getPreviousUrl();
    console.log("redirect_login: " + this.globalService.redirect_login);
       
    
  } 

  createAccount() {
    this.router.navigateByUrl('/cadastro-usuario');
  }
  
  resetPassword() {
    //this.navCtrl.push(ResetpasswordPage);
  }

  login() {

    if (this.form.valid) {
      this.authService.login(UtilsProvider.removeMascara(this.form.controls["CPF"].value), this.form.controls["EMAIL"].value, this.globalService.redirect_login)
      .then(() => {
        
        console.log('Login OK!');
        
      })
      .catch((error: any) => {
        this.msg.showAlert('CPF ou Email Inválidos!');          
      });

    }else{
      let vlmessagem: string = 'Preencha o campo ';
      const controls = this.form.controls;
      for (const name in controls) {
        if (controls[name].invalid) {
          vlmessagem = vlmessagem + name;
          break;
        }
      }
      
      this.msg.showToast(vlmessagem);      
    }

  }
  
}
