import { Component, ViewChild, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Events, LoadingController } from '@ionic/angular';
import { GlobalService } from '../../services/global.service';
import { AuthService } from '../../services/auth/auth-service';
import { ApiProvider } from '../../services/api/api';
import { MsgProvider } from '../../services/msg/msg';
import { UtilsProvider } from '../../services/utils/utils';
import { User } from '../../model/user';
import { ConfiguracaoProvider } from '../../services/configuracao/configuracao';
import { NavigationEnd, Router } from '@angular/router';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';

/**
 * Generated class for the MeusdadosPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@Component({
  selector: 'app-meus-dados',
  templateUrl: './meus-dados.page.html',
  styleUrls: ['./meus-dados.page.scss'],

})
export class MeusDadosPage implements OnInit, OnEnter{
  subscription: Subscription;
  form: FormGroup;
  @ViewChild('inputCPF') inputCPF;
  // angularForm = new FormGroup ({
  //   name: new FormControl()
  // });

  @ViewChild('inputNUMERO') inputNUMERO; 

  ufs = [];
  cidades = [];  

  constructor(public globalService: GlobalService,
              private formBuilder: FormBuilder,
              private api: ApiProvider,              
              private router: Router, 
              private msg: MsgProvider,
              public authService: AuthService,
              public loadingCtrl: LoadingController,
              private config: ConfiguracaoProvider) {

     this.createForm();      
  }

  public async onEnter(): Promise<void> {
    console.log("OnEnter MeusDadosPage");
    
    this.atualizaMeusDados();
    
  } 

  public async ngOnInit(): Promise<void> {
      await this.onEnter();

      this.subscription = this.router.events.subscribe((event:any) => {
          if (event instanceof NavigationEnd && event.url === '/meus-dados') {
              this.onEnter();
          }
      });
  }  
  

  atualizaMeusDados(){
    console.log("atualizaMeusDados");
    if (this.authService.logado()){      
      this.form.controls["CODIGO_CLIENTE"].setValue(this.authService.usuarioLogado().CODIGO_CLIENTE);
      this.form.controls["NOME"].setValue(this.authService.usuarioLogado().NOME);
      this.form.controls["EMAIL"].setValue(this.authService.usuarioLogado().EMAIL);
      this.form.controls["CPF"].setValue(UtilsProvider.mascaraCPF(this.authService.usuarioLogado().CPF));
      //this.inputCPF.value = this.authService.usuarioLogado().CPF;
      this.form.controls["CELULAR"].setValue(UtilsProvider.mascaraCelular(this.authService.usuarioLogado().CELULAR));
      this.form.controls["CEP"].setValue(UtilsProvider.mascaraCEP(this.authService.usuarioLogado().CEP));
      this.form.controls["ENDERECO"].setValue(this.authService.usuarioLogado().ENDERECO);
      this.form.controls["NUMERO"].setValue(this.authService.usuarioLogado().NUMERO);
      this.form.controls["COMPLEMENTO"].setValue(this.authService.usuarioLogado().COMPLEMENTO);
      this.form.controls["BAIRRO"].setValue(this.authService.usuarioLogado().BAIRRO);
      this.form.controls["CIDADE"].setValue(this.authService.usuarioLogado().CIDADE);
      this.form.controls["UF"].setValue(this.authService.usuarioLogado().UF);
      this.form.controls["CODIGO_CEP_CIDADE"].setValue(this.authService.usuarioLogado().CODIGO_CEP_CIDADE);
      this.form.controls["CODIGO_CEP_UF"].setValue(this.authService.usuarioLogado().CODIGO_CEP_UF);
    }   
  }

  
  createForm() {

    this.form = this.formBuilder.group({
      CODIGO_FRANQUIA: this.formBuilder.control('', [Validators.required]),
      CODIGO_CLIENTE: this.formBuilder.control('', [Validators.required]),
      NOME: this.formBuilder.control('', [Validators.required]),
      EMAIL: this.formBuilder.control('', [Validators.required, Validators.minLength(5)]),
      CPF: this.formBuilder.control('', [Validators.required, Validators.minLength(11)]),
      CELULAR: this.formBuilder.control('', [Validators.required]),
      CEP: this.formBuilder.control('', [Validators.required]),
      ENDERECO: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      NUMERO: this.formBuilder.control('', [Validators.required]),
      COMPLEMENTO: this.formBuilder.control(''),
      BAIRRO: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_CEP_CIDADE: this.formBuilder.control('', [Validators.required]),
      CIDADE: this.formBuilder.control({value: '', disabled: true}, [Validators.required]),
      CODIGO_CEP_UF: this.formBuilder.control('', [Validators.required]),
      UF: this.formBuilder.control({value: '', disabled: true}, [Validators.required])
    })
     
  }

  irParaLogin(){
    this.router.navigateByUrl('/login');    
  }

  onSubmit() {
    this.form.controls["CODIGO_FRANQUIA"].setValue(this.globalService.franquia.CODIGO_FRANQUIA);

    if (this.form.valid) {

          let usuario: User = new User();
          usuario.CODIGO_CLIENTE = this.form.controls['CODIGO_CLIENTE'].value;
          usuario.CODIGO_FRANQUIA = this.form.controls['CODIGO_FRANQUIA'].value;
          usuario.CPF = UtilsProvider.removeMascara(this.form.controls["CPF"].value);
          usuario.NOME = this.form.controls["NOME"].value;
          usuario.EMAIL  = this.form.controls["EMAIL"].value;      
          usuario.CELULAR = UtilsProvider.removeMascara(this.form.controls["CELULAR"].value);
          usuario.CEP = UtilsProvider.removeMascara(this.form.controls["CEP"].value);
          usuario.ENDERECO = this.form.controls["ENDERECO"].value;
          usuario.BAIRRO = this.form.controls["BAIRRO"].value;
          usuario.NUMERO = this.form.controls["NUMERO"].value;
          usuario.COMPLEMENTO = this.form.controls["COMPLEMENTO"].value;        
          usuario.CODIGO_CEP_CIDADE = this.form.controls["CODIGO_CEP_CIDADE"].value;        
          usuario.CODIGO_CEP_UF = this.form.controls["CODIGO_CEP_UF"].value;

        this.api.cadastrarUsuario(usuario)
        .then(() => {              
          //Atualiza usuario no dispositivo 
          this.authService.setUsuarioLogado(usuario);
          //Atualiza no banco local
          this.config.gravaUsuario(this.authService.usuarioLogado());
          
          this.msg.showToast('Usuário salvo com sucesso.');

        })
        .catch((e) => {
          this.msg.showToast('Erro ao salvar o usuário.');
          console.error(e);
        })               
    }else{
      let vlmessagem: string = 'Preencha o campo ';
      const controls = this.form.controls;
      for (const name in controls) {
        if (controls[name].invalid) {
          vlmessagem = vlmessagem + name;
          break;
        }
      }
      
      this.msg.showToast(vlmessagem);
    }
  }    
  consultaCEP(){
    if (this.form.controls['CEP'].valid){
      this.api.consultaCEP(UtilsProvider.removeMascara(this.form.controls['CEP'].value))
      .then(
        (dados: any) =>{
          if (UtilsProvider.isEmptyObj(dados)){
            this.msg.showToast('CEP não localizado!');
          }else{
            this.form.controls['ENDERECO'].setValue(dados[0].ENDERECO);
            this.form.controls['BAIRRO'].setValue(dados[0].BAIRRO);
            this.form.controls['CODIGO_CEP_CIDADE'].setValue(dados[0].CODIGO_CEP_CIDADE);
            this.form.controls['CIDADE'].setValue(dados[0].CIDADE);
            this.form.controls['CODIGO_CEP_UF'].setValue(dados[0].CODIGO_CEP_UF);
            this.form.controls['UF'].setValue(dados[0].UF);
            this.inputNUMERO.focus();
          }
        }
      )
      .catch(
        (erro)=>{
          this.msg.showToast('Erro ao localizar cep. '+ erro.message);
        }
      );
    }else{
      this.msg.showToast('Informe um cep Válido!');
    }
  }

  lerTermo(){
    this.router.navigateByUrl('/termo-uso');
  }  
}
