import { Component, OnInit } from '@angular/core';
import { Events, LoadingController } from '@ionic/angular';
import { GlobalService } from '../../services/global.service';
import { ApiProvider } from '../../services/api/api';
import { AuthService } from '../../services/auth/auth-service';
import { MsgProvider } from '../../services/msg/msg';
import { NavigationEnd, Router } from '@angular/router';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';

/**
 * Generated class for the MeuspedidosPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@Component({
  selector: 'app-meus-pedidos',
  templateUrl: './meus-pedidos.page.html',
  styleUrls: ['./meus-pedidos.page.scss'],
})
export class MeusPedidosPage implements OnInit, OnEnter {  
  pedidos: string[];
  errorMessage: string;
  loader: any;
  subscription: Subscription;

  constructor(public globalService: GlobalService,
              public authService: AuthService,
              public loadingCtrl: LoadingController,
              private api: ApiProvider,              
              private msg: MsgProvider,
              private router: Router) {
      this.pedidos = [];                
  }

  public async onEnter(): Promise<void> {
    console.log("OnEnter MeusPedidosPage");
    
    this.atualizaPedidos(true);
    
  } 

  public async ngOnInit(): Promise<void> {
      await this.onEnter();

      this.subscription = this.router.events.subscribe((event:any) => {
          if (event instanceof NavigationEnd && event.url === '/tabs/meus-pedidos') {
              this.onEnter();
          }
      });
  }

  // ionViewDidEnter() {
  //   console.log('ionViewDidEnter 1');        
  // }

  doRefresh(refresher){
    if (this.authService.logado()){
      this.api.getPedidos(this.authService.userId())
          .subscribe(
            pedidos => {
              this.pedidos = pedidos;
              refresher.target.complete();
            },
            error =>  {
              this.errorMessage = <any>error
            });      
    }else{
      refresher.target.complete();
    }        
  }  

  async atualizaPedidos(showLoad: boolean){
    if (this.authService.logado()){
      if (showLoad){
        this.loader = await this.loadingCtrl.create({
          message: "Atualizando pedidos..."
        });      
        this.loader.present();        
      }
      console.log('Atualizando pedidos...');

      this.api.getPedidos(this.authService.userId())
            .subscribe(
              pedidos => {
                this.pedidos = pedidos;
                if (showLoad){
                  this.loader.dismiss();
                }  
              },
              error => {
                if (showLoad){
                  this.loader.dismiss();
                }
                this.msg.showToast(<any>error)
              } 
            );
    }else{
        this.pedidos = []; 
    }    
  }

  ionViewDidEnter() {
    
    
  }

  irParaLogin(){
    this.router.navigateByUrl('/login');
  }

  irParaBrinquedos(){
    this.router.navigateByUrl('/tabs/brinquedos');    
  }

  async irParaDetalhe(pedido){
    let loader = await this.loadingCtrl.create({
      message: "Aguarde Um momento..."
    });
    loader.present();
    this.api.getPedidoDetalhe(pedido.CODIGO_PEDIDO).then(
      (resposta: any) => {
        loader.dismiss();        
        
        this.globalService.detalhe_pedido_pedido = pedido;    
        this.globalService.detalhe_pedido_itens = resposta.itens;   
        this.globalService.detalhe_pedido_historicos = resposta.historicos;
        this.globalService.detalhe_pedido_comprovante = resposta.comprovante;

        this.router.navigateByUrl('/detalhe-pedido');        
      }
    ).catch(
      () => {
        loader.dismiss();
      }  
    )
    
    

    
  }
}
