import {Component, NgZone, OnInit} from '@angular/core';
import { ActionSheetController,Platform, LoadingController } from '@ionic/angular';
import { CarrinhoLocalProvider } from '../../services/carrinho/carrinhoLocal';

import { MsgProvider } from '../../services/msg/msg';
import { GlobalService } from '../../services/global.service';
import {ApiProvider} from "../../services/api/api";
import { environment } from '../../../environments/environment';
import { UtilsProvider } from '../../services/utils/utils';
import { OnEnter } from '../../interface/on-enter';
import { Subscription } from 'rxjs/Subscription';
import { NavigationEnd, Router } from '@angular/router';


declare var PagSeguroDirectPayment:any;

@Component({
  selector: 'app-pagseguro',
  templateUrl: './pagseguro.page.html',
  styleUrls: ['./pagseguro.page.scss'],
})
export class PagseguroPage implements OnInit, OnEnter {

  imagens_pagseguro: string;  
  subscription: Subscription;

  paymentMethod = 'CREDIT_CARD';
  //paymentMethods: Array<any> = [];

  //### TESTE
  // creditCard = {
  //     num: '4111111111111111',
  //     name: 'osman setuval',
  //     cvv: '123',
  //     monthExp: '12',
  //     yearExp: '2030',
  //     brand: '',
  //     token: '',
  //     parcela: '',
  //     parcelas: []
  // };

  creditCard = {
    num: '',
    name: '',
    cvv: '',
    monthExp: '',
    yearExp: '',
    brand: '',
    token: '',
    parcela: '',
    parcelas: [],
    data_nascimento: ''
  };


  constructor(public carrinho: CarrinhoLocalProvider,              
    private msg: MsgProvider,
    public globalService: GlobalService,
    public actionSheetCtrl: ActionSheetController,
    public platform: Platform,
    public loadingCtrl: LoadingController,    
    public api: ApiProvider,
    public zone: NgZone,
    private router: Router) {

            
    this.imagens_pagseguro = environment.imagens_pagseguro;
  }

  public async ngOnInit(): Promise<void> {
    console.log("pagseguroPage ngOnInit");
    await this.onEnter();

    this.subscription = this.router.events.subscribe((event:any) => {
        if (event instanceof NavigationEnd && event.url === '/pagseguro') {
            this.onEnter();
        }
    });
  }  

  public async onEnter(): Promise<void> {
    console.log("OnEnter PagseguroPage");
    this.iniciaSessaoPagseguro();    
    
  }   

  initSession(data) {
    console.log("Iniciou sessão:" + data.sessionId);
    PagSeguroDirectPayment.setSessionId(data.sessionId);
  }
  
  // getPaymentMethods() {
  //     PagSeguroDirectPayment.getPaymentMethods({
  //         amount: 100,
  //         success: (response) => {
  //             this.zone.run(() => {
  //                 let paymentMethods = response.paymentMethods;
  //                 this.paymentMethods = Object.keys(paymentMethods).map((key) => paymentMethods[key]);
  //                 setTimeout(() => {
  //                     this.segment._inputUpdated();
  //                     this.segment.ngAfterContentInit();
  //                 });
  //             });
  //         }
  //     });
  // }
  
  onChangeCartao() : void {
  
    this.getCreditCardBrand().then(
      () =>{
        console.log("onChangeCartaoe BANDEIRA:" + this.creditCard.brand);
        this.buscaParcelas();

      }
    )
    
  }
  
  async makePayment() {
  
    if (UtilsProvider.empty(this.creditCard.num)){
      this.msg.showAlert("Preecha o Número do cartão");
      return;
    }
  
    if (UtilsProvider.empty(this.creditCard.name)){
      this.msg.showAlert("Preecha o Nome (como gravado no cartão)");
      return;
    }

    if (UtilsProvider.empty(this.creditCard.name)){
      this.msg.showAlert("Preecha a Data de Nascimento");
      return;
    }


    if (UtilsProvider.empty(this.creditCard.cvv)){
      this.msg.showAlert("Preecha o Código de segurança");
      return;
    }
  
    if (UtilsProvider.empty(this.creditCard.monthExp)){
      this.msg.showAlert("Preecha o Mês de Validade");
      return;
    }
  
    if (UtilsProvider.empty(this.creditCard.yearExp)){
      this.msg.showAlert("Preecha o Ano da Validade");
      return;
    }
    
    if (UtilsProvider.empty(this.creditCard.parcela)){
      this.msg.showAlert("Preecha o Parcelamento");
      return;
    }


    let loader = await this.loadingCtrl.create({
      message: "Processando Pedido..."
    });
    loader.present();

    var str1:string = this.creditCard.parcela;
    var index = str1.indexOf("|"); 
    var parcela = str1.substring(1,index-1);
    var valor_parcelado = str1.substring(index+1, str1.length);

    console.log("#### DATA NASCIMENTO: "+this.creditCard.data_nascimento);

    let data = {
        items: this.carrinho.itens,
        cliente: this.carrinho.cliente,        
        codigo_franquia: this.globalService.franquia.CODIGO_FRANQUIA,
        pedido: this.carrinho.pedido, 
        hash: PagSeguroDirectPayment.getSenderHash(),
        method: this.paymentMethod,
        parcela: parcela,
        valor_parcelado: valor_parcelado,
        desconto: -1*this.carrinho.desconto,
        total: this.carrinho.valor_total,
        data_nascimento: this.creditCard.data_nascimento
    };
  
    let doPayment = () => {
        this.api.pagseguroPagamento(data).subscribe(
          (data:any) => {
            if (data.possui_erro){
              loader.dismiss();
              this.msg.showAlert('Erro ao processar pagamento. Codigo: ' +data.codigo_erro+ ' Erro: ' + data.messagem_erro);
            }else{
              console.log("FIM DE PAGAMENTO  CODE:"+data.codigo_pagseguro);
              loader.dismiss();
              this.msg.showAlert("Processamento concluí­do com sucesso!");
              this.finalizarCarrinho();          
            }

            
          },
        
          (err) => {
            loader.dismiss();
            this.msg.showAlert('Erro ao processar pagamento. Erro: ' + err.message);
          }
        );
    };
    if(this.paymentMethod == 'CREDIT_CARD'){
      this.prepareCreditCard().
      then(() => {
          (<any>data).token = this.creditCard.token;
          console.log("Cartão Token: " + (<any>data).token);
          doPayment();
      },(error) => {
          console.log(error);
          loader.dismiss();
          this.msg.showAlert("Erro ao gerar token do cartão. Erro: " + error.message);
      })
      .catch(
          (error) =>{      
            console.log(error);  
            loader.dismiss();  
            this.msg.showAlert("Erro ao gerar token do cartão. Erro: " + error.message);
          }
      );
    }  
  }
  
  prepareCreditCard(): Promise<any> {
    return this.getCreditCardBrand().then(() => {
        return this.getCreditCardToken();
    });
  }
  
  getCreditCardBrand(): Promise<any> {
    return new Promise((resolve,reject) => {
        PagSeguroDirectPayment.getBrand({
            cardBin: this.creditCard.num.substring(0, 6),
            success: (response) => {
                this.zone.run(() => {
                    this.creditCard.brand = response.brand.name;
                    console.log("BANDEIRA CARTAO: " + response.brand.name);                    
                    resolve({brand: response.brand.name});
                });
            },
            error(error){
                reject(error)
            }
        });
    });
  }
  
  getCreditCardToken(): Promise<any> {
    return new Promise((resolve,reject) => {
        PagSeguroDirectPayment.createCardToken({
            cardNumber: this.creditCard.num,
            brand: this.creditCard.brand,
            cvv: this.creditCard.cvv,
            expirationMonth: this.creditCard.monthExp,
            expirationYear: this.creditCard.yearExp,
            success: (response) => {
                this.zone.run(() => {
                    this.creditCard.token = response.card.token;
                    resolve({token: response.card.token});
                    console.log(response);
                });
            },
            error(error){
                reject(error)
            }
        })
    });
  }

  buscaParcelas(){
   console.log("buscaParcelas...");

    PagSeguroDirectPayment.getInstallments({

				amount:  this.carrinho.valor_total,              //valor total da compra (deve ser informado)
				brand: this.creditCard.brand, //bandeira do cartão (capturado na função buscaBandeira)
				//maxInstallmentNoInterest: 3,
				success: response => { 
				    this.zone.run(() => {
              console.log("buscaParcelas success");
              this.creditCard.parcelas = response.installments[this.creditCard.brand];
              console.log('parcelas: '+ response);
            });
				},
				error: response => {	console.log( response)	}
		});

  }
  
  async iniciaSessaoPagseguro(){
    if (this.globalService.getStatusPagSeguro){
      let loader = await this.loadingCtrl.create({
        message: "Iniciando Pagseguro..."
      });
      await loader.present();
      this.api.pagseguroSessao(this.globalService.franquia.CODIGO_FRANQUIA)
                .subscribe(
                  data => {
                    console.log("Iniciou sessão: "+data.sessionId)
                    this.initSession(data); 
                    loader.dismiss();          
                  },
                  erro =>{                
                    loader.dismiss();
                    this.msg.showAlert('Erro ao iniciar PagSeguro! Erro: '+erro.message);
                  }
                )
    }else{
      this.msg.showAlert('Pagseguro não carregado!');
    }            
  }
  
  possuiBandeira(){
   
    if (UtilsProvider.empty(this.creditCard.brand)){
      return false;
    }else{
      return true;
    }   
  }  


  finalizarCompra(){
    this.makePayment();    
  }
  
  finalizarCarrinho(){
    //Dispara Evento que Limpa o carrinho
    this.carrinho.limpaCarrinho()
    .then(
      () => {
        this.irParaTabPedidos();    
      }
    );      
  }

  irParaTabPedidos(){
    this.router.navigateByUrl('/tabs/meus-pedidos');
  }  

}
