import { Component, OnInit  } from '@angular/core';
import { LoadingController, Events } from '@ionic/angular';
import { ConfiguracaoProvider } from '../../services/configuracao/configuracao';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { GlobalService } from '../../services/global.service';
import { environment } from '../../../environments/environment';
import { ApiProvider } from './../../services/api/api';
import { MsgProvider } from '../../services/msg/msg';
import { Router } from '@angular/router';
import { Franquia } from '../../model/franquia';
/**
 * Generated class for the SelecionaFranquiaPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@Component({
  selector: 'app-seleciona-franquia',
  templateUrl: './seleciona-franquia.page.html',
  styleUrls: ['./seleciona-franquia.page.scss'],
})
export class SelecionaFranquiaPage implements OnInit {
  
  form: FormGroup;
  franquias = [];
  versao: string;
  //loader: any;

  constructor(private formBuilder: FormBuilder, 
              private configuracao: ConfiguracaoProvider,
              private globalService: GlobalService,
              public loadingCtrl: LoadingController,
              private api :ApiProvider,
              private msg: MsgProvider,
              private router: Router,
              public events: Events
            ) {
     this.versao = environment.versao;
     this.createForm();
  }

  ngOnInit() {
  }

  ionViewDidEnter() {
    this.atualizarFranquias();
  }

  createForm() {
    this.form = this.formBuilder.group({
      codigo_franquia: this.formBuilder.control('', [Validators.required])
    })
  }

  onSubmit() {
    console.log("onSubmit");
    if (this.form.valid) {
      
      
      let codigo = this.form.controls['codigo_franquia'].value;
      
      let achou: boolean = false;

      for (var i = 0; i < this.franquias.length; i++){
        var franquia = this.franquias[i];
        if (franquia.CODIGO_FRANQUIA == codigo){
          //Salva no Banco Local
          achou = true;

          this.api.getFranquia(franquia.CODIGO_FRANQUIA)
          .then(
            (franquiaServer: Franquia) => {
              console.log("Dados da Franquia Atualizado com servidor!");
              this.globalService.franquia = franquiaServer;
              this.configuracao.setFranquia(franquiaServer);
              this.router.navigateByUrl('/tabs');            
            }
          ) 
          .catch(
            (erro) =>{
              this.msg.showAlert('Falha ao carregar dados da franquia, Verifique sua conexão e tente novamente!')      
            }
          )                   
        }
      }

      if (!achou){
        this.globalService.franquia = null;
        this.configuracao.setFranquia(null);
        this.msg.showAlert('Franquia não localizada!')
      }
    }else{
      let vlmessagem: string = 'Meu Erro';
      if (!this.form.controls['codigo_franquia'].valid){
        vlmessagem = 'Selecione a cidade!'
      }

      this.msg.showToast(vlmessagem);      
    }    

  }

  async atualizarFranquias(){
    let loader = await this.loadingCtrl.create({
      message: "Carregando Franquias..."
    });      
    loader.present();

    console.log("atualizaFranquias INICIO");
    this.api.getFranquias()
      .then((result: any) => {
        this.franquias = [];
        if (result){    
          console.log("atualizaFranquias result:"+result[0].CODIGO_FRANQUIA);
          this.franquias = result;                
          loader.dismiss();
        }else{
          loader.dismiss();
          this.msg.showAlertErro("Nenhuma franquia foi carregada!")
        }  
      })
      .catch(        
        (error) => {
          loader.dismiss();
          this.msg.showAlertErro(error);
        }  
      );    
  }

}