import { HttpClient , HttpHeaders} from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from '../../../environments/environment';
import 'rxjs/add/operator/map';
import { map, catchError } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { throwError } from 'rxjs';
import { UtilsProvider } from '../../services/utils/utils';
import { User } from '../../model/user';

/*
  Generated class for the ApiProvider provider.

  See https://angular.io/guide/dependency-injection for more info on providers
  and Angular DI.
*/
@Injectable({
  providedIn: 'root'
})
export class ApiProvider {

  constructor(public http: HttpClient) {
    console.log('Hello ApiProvider Provider');
  }

  montaHeader():HttpHeaders{
    //let lstring = environment.api_user + ":" + environment.api_s;
    //console.log("montaHeader : "+ lstring);
    // Encode the String
    //var encodedString = btoa(lstring);
    //var encodedString = Base64.encode(environment.api_user + ':' + environment.api_s)
    //console.log("montaHeader encodedString: "+ encodedString);
    
  
    let headers = new HttpHeaders();
    headers = headers.append('Access-Control-Allow-Origin' , '*');
    //headers = headers.append('Access-Control-Allow-Headers','X-Requested-With, Content-Type, Accept, Origin, Authorization');
    //headers = headers.append('Access-Control-Allow-Methods', 'POST, GET, OPTIONS, PUT');
    //headers = headers.append('Accept','application/json');
    //headers = headers.append('Access-Control-Allow-Credentials','true');    
    //headers = headers.append('content-type','application/json, application/x-www-form-urlencoded');    
    headers = headers.append('content-type','application/json');        
    headers = headers.append('Authorization', "Basic "+  btoa(environment.api_user + ":" + environment.api_s));
    return headers;
  }


  checkinApp(codigo_franquia: string, 
              cpf: string,               
              versao: string,
              token: string){
    let headers = this.montaHeader();
    return new Promise((resolve, reject) => {
      
      let url = environment.api + 'app/checkin';
      const body = {
        CODIGO_FRANQUIA: codigo_franquia,
        CPF: cpf,
        VERSAO: versao,
        TOKEN: token
      };

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });    
  }

  versaoAtualAplicativo(){
    let headers = this.montaHeader();
    return new Promise((resolve, reject) => {

    let url = environment.api + 'app/versao-atual';
    
    this.http.get(url, {headers: headers})
      .subscribe((result: any) => {
        resolve(result);
      },
      (error) => {
        reject(error);
      });
    });    
  }

  
  pagseguroSessao(codigo_franquia: string): Observable<any> {
    
    let headers = this.montaHeader();

    return this.http.get(environment.api + 'pagseguro/sessao/' + codigo_franquia, {headers: headers}).pipe(
      map(this.extractData),
      catchError(this.handleError)
    );
     //.map(response => response.json());
  }

  pagseguroPagamento(data): Observable<Object> {
      let headers = this.montaHeader();

      return this.http.post(environment.api + 'pagseguro/pagamento', data, {headers: headers}).pipe(
        map(this.extractData),
        catchError(this.handleError)
      );
        //  .map(response => response.json());
  }

  salvaCarrinho(dados) {
    return new Promise((resolve, reject) => {
      
      let headers = this.montaHeader();

      let url = environment.api + 'carrinho';      

      this.http.post(url, dados, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  revalidarCarrinho(dados) {
    return new Promise((resolve, reject) => {
      
      let headers = this.montaHeader();

      let url = environment.api + 'revalidar-carrinho';      

      this.http.post(url, dados, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  getFranquias() {
    return new Promise((resolve, reject) => {

      let headers = this.montaHeader();

      let url = environment.api + 'franquias';
      
      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          console.log(error.headers)
          reject(this.trataErro(error));
        });
    });
  }

  getFranquia(codigo_franquia) {
    return new Promise((resolve, reject) => {

      let headers = this.montaHeader();

      let url = environment.api + 'franquias/'+codigo_franquia;
      
      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          console.log(error.headers)
          reject(this.trataErro(error));
        });
    });
  }

  getFranquiaContas(codigo_franquia) {
    return new Promise((resolve, reject) => {

      let headers = this.montaHeader();

      let url = environment.api + 'franquias-contas/'+codigo_franquia;
      
      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          console.log(error.headers)
          reject(this.trataErro(error));
        });
    });
  }
  //: Observable<string[]>
  getPedidos(userId: string):Observable<any> {
    let headers = this.montaHeader();

    return this.http.get(environment.api + 'pedidos/' + userId, {headers: headers}).pipe(
      map(this.extractData),
      catchError(this.handleError)
    );
  }

  getPedidos1(userId: string) {
    return new Promise((resolve, reject) => {

      let headers = this.montaHeader();
      let url = environment.api + 'pedidos/' + userId;

      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  anexarComprovante(dados) {
    let headers = this.montaHeader();
    return new Promise((resolve, reject) => {

      let url = environment.api + 'anexar-comprovante';      

      this.http.post(url, dados, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  getPedidoDetalhe(codigo_pedido: string) {
    let headers = this.montaHeader();
    return new Promise((resolve, reject) => {

      let url = environment.api + 'pedido_detalhe/' + codigo_pedido;

      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }  

  //###PRODUTOS
  //: Observable<string[]>
  getProdutos(codigo_franquia: string):Observable<any> {
    let headers = this.montaHeader();
    console.log("getProdutos codigo_franquia:"+codigo_franquia);
    const body = {
      CODIGO_FRANQUIA: codigo_franquia
    };
    console.log("URL: "  + environment.api + 'produtos')
    return this.http.post(environment.api + 'produtos', body,{headers: headers})
    .pipe(
      map(this.extractData),
      catchError(this.handleError)
    );
  }

  getProdutosPaginacao(codigo_franquia: string, 
                       page: number,
                       filtro_status: string,
                       filtro_titulo: string) {
    return new Promise((resolve, reject) => {

      const body = {
        CODIGO_FRANQUIA: codigo_franquia,
        PAGINA: page,
        LIMITE: 10,
        FILTRO_STATUS: filtro_status,
        FILTRO_TITULO: filtro_titulo
      };
      let url = environment.api + 'produtos-paginacao';
      const headers = this.montaHeader();
      
      this.http.post(url, body, {headers: headers} )
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }
  
  produtoIndisponivel(produto, usuario, codigo_franquia) {
    let headers = this.montaHeader();
    return new Promise((resolve, reject) => {
      
      let url = environment.api + 'produto_indisponivel';
      const body = {
        CODIGO_FRANQUIA: codigo_franquia,
        CODIGO_PRODUTO: produto.CODIGO_PRODUTO,
        NOME: usuario.NOME,
        EMAIL: usuario.EMAIL,
        CELULAR: usuario.CELULAR
      };

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  getLogin(cpf: string, email: string) {
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'clientes/login';
      const body = {cpf: cpf, email: email};

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  atualizaToken(codigo_cliente: string, token: string) {
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'clientes/atualiza-token';
      const body = {CODIGO_CLIENTE: codigo_cliente, TOKEN: token};

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }
  
  recuperarSenha(email: string) {
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'clientes/recupera_senha';
      const body = {EMAIL: email};

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }
  
  cadastrarUsuario(usuario: User) {
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'clientes';
      const body = {
        CODIGO_CLIENTE: usuario.CODIGO_CLIENTE,
        CODIGO_FRANQUIA: usuario.CODIGO_FRANQUIA,
        CPF: usuario.CPF,
        NOME: usuario.NOME,
        EMAIL: usuario.EMAIL,        
        CELULAR: usuario.CELULAR,
        CEP: usuario.CEP,
        ENDERECO: usuario.ENDERECO,
        BAIRRO: usuario.BAIRRO,
        NUMERO: usuario.NUMERO,
        COMPLEMENTO: usuario.COMPLEMENTO,        
        CODIGO_CEP_CIDADE: usuario.CODIGO_CEP_CIDADE,        
        CODIGO_CEP_UF: usuario.CODIGO_CEP_UF
      };

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  existeCPF(cpf){
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'cliente-codigo/'+cpf;
    

      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }  
  
  consultaCEP(cep){
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'consulta-cep/'+cep;
    

      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }  

  getEstados() {
    return new Promise((resolve, reject) => {

      let headers = this.montaHeader();

      let url = environment.api + 'estados';
      
      this.http.get(url, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          console.log(error.headers)
          reject(error);
        });
    });
  }


  getCupom(codigo_franquia, cupom){
    let headers = this.montaHeader();

    return new Promise((resolve, reject) => {

      let url = environment.api + 'carrinho/cupom';
      const body = {CODIGO_FRANQUIA: codigo_franquia, CUPOM: cupom};

      this.http.post(url, body, {headers: headers})
        .subscribe((result: any) => {
          resolve(result);
        },
        (error) => {
          reject(error);
        });
    });
  }

  private extractData(res: Response) {
    console.log("extractData");
    let body = res;
    return body || {};
  }

  private handleError (error: Response | any) {
  
    // console.log("handleError error.status: " + error.status);
    // console.log("handleError error.statusText: " + error.statusText);
    // let body = error;
    // console.log("handleError error: " + error);

    // var propValue;
    // for(var propName in error) {
    //   propValue = error[propName]

    //   console.log("handleError propName: "+ propName,"propValue: " +propValue);
    // }
    //console.log("handleError error: " + error.content);
    //of(error.json())

    // if (error instanceof Response) {
      
    //   const err = error || '';
    //   errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
    // } else {
    //   console.error("Entrou AQUI2");
    //   errMsg = error.message ? error.message : error.toString();
    // }    

    let errMsg: string;
    if (error.status === 0){
      errMsg = 'Falha na comunicação com o Servidor';
    }else if (error.status === 400 || error.status === 500){
      errMsg = 'Falha no Servidor';
    }else{
      errMsg = `${error.status} - ${error.statusText || ''} ${error.toString()}`;
    }  
    
    console.log("handleError errMsg: " + errMsg);
    return throwError(new Error(errMsg));     
  }  

  trataErro(error){
    let errMsg: string;
    if (error.status === 0){
      errMsg = 'Aplicativo não consegue conectar ao servidor. Problema de CORS!';
    }else {
      errMsg = `${error.status} - ${error.statusText || ''} ${error}`;
    }  
    return errMsg;
  }
}
