import { Injectable } from '@angular/core';
//import { AngularFireAuth } from 'angularfire2/auth';
import { User } from '../../model/user';
import { ApiProvider } from '../api/api';
import { Events } from '@ionic/angular';
import { ConfiguracaoProvider } from '../../services/configuracao/configuracao';

import { Router } from '@angular/router';
import { CarrinhoLocalProvider } from '../../services/carrinho/carrinhoLocal';
import { GlobalService } from '../../services/global.service';

//import * as firebase from 'firebase/app';
// import { GooglePlus } from '@ionic-native/google-plus';
// import { Facebook, FacebookLoginResponse } from '@ionic-native/facebook';
// import { TwitterConnect } from '@ionic-native/twitter-connect';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  private _usuarioLogado:User;
  
  ultimaRotaLogin: string;

  constructor(private api: ApiProvider, 
              public events: Events,
              private config: ConfiguracaoProvider,
              private router: Router,
              private carrinho: CarrinhoLocalProvider,
              private globalService: GlobalService) { }

  createUser(user: User) {
    //return this.angularFireAuth.auth.createUserWithEmailAndPassword(user.email, user.password);
  }

  usuarioLogado():User {
    return this._usuarioLogado;
  }

  setUsuarioLogado(user :User) {
    this._usuarioLogado = user;
    if (user && this.globalService.TOKEN){
      console.log("ATUALIZANDO TOKEN...");
      this.api.atualizaToken(user.CODIGO_CLIENTE, this.globalService.TOKEN);
    }
    
    if (user){
      console.log("### CHECKINAPP");
      this.api.checkinApp(
        this.globalService.franquia.CODIGO_FRANQUIA,
        user.CPF,
        this.globalService.versao,
        this.globalService.TOKEN      
      );
    }  
  }

  getUsuarioLocal(){
    return new Promise((resolve, reject) => {
    
      this.config.getUsuario()
      .then(
        (usuario: User) =>{          
          resolve(usuario);  
        }
      )
      .catch(
        (error) => {
          console.log("Falha ao carregar usuario LOCAL: " + error.message);
          reject(error);
        }  
      );
    });   
  }

  login(cpf: string, email: string, redirect: string ="") {    
    return new Promise((resolve, reject) => {      
      this.api.getLogin(cpf, email)
      .then(
        (usuario: User) => {
          this.setUsuarioLogado(usuario);
          console.log("### LOGIN ");
          console.log("  CODIGO_CLIENTE : " + usuario.CODIGO_CLIENTE);
          console.log("  CPF : " + usuario.CPF);
          console.log("  ENDERECO : " + usuario.ENDERECO);
          this.config.gravaUsuario(usuario);

          if (redirect){
            this.router.navigateByUrl(redirect);
          }

          resolve(usuario);
        } 
      )
      .catch(
        (error) => {
          console.log("### ERRO LOGIN : " +error.message);
          
          reject(error);
        }
      );
    });  
  }


  signOut() {
    
    this._usuarioLogado = null;

    return Promise.all([
      this.config.removeFranquia(),
      this.config.removeUsuario(),
      this.carrinho.limpaCarrinho(),
      this.events.publish('app:fechar')
    ]);
        
    
    // if (this.angularFireAuth.auth.currentUser.providerData.length) {
    //   for (var i = 0; i < this.angularFireAuth.auth.currentUser.providerData.length; i++) {
    //     var provider = this.angularFireAuth.auth.currentUser.providerData[i];

    //     if (provider.providerId == firebase.auth.GoogleAuthProvider.PROVIDER_ID) { // Se for o gooogle
    //       // o disconnect limpa o oAuth token e tambem esquece qual conta foi selecionada para o login
    //       return this.googlePlus.disconnect()
    //         .then(() => {
    //           return this.signOutFirebase();
    //         });
    //     } else if (provider.providerId == firebase.auth.FacebookAuthProvider.PROVIDER_ID) { // Se for facebook
    //       return this.facebook.logout()
    //         .then(() => {
    //           return this.signOutFirebase();
    //         })
    //     } else if (provider.providerId == firebase.auth.TwitterAuthProvider.PROVIDER_ID) { // Se for twitter
    //       return this.twitter.logout()
    //         .then(() => {
    //           return this.signOutFirebase();
    //         })
    //     }
    //   }
    // }

    // return this.signOutFirebase();
  }

  
  logado(): boolean {
    var logado: boolean;
    if (this._usuarioLogado){
      logado = true
    }else{
      logado = false
    }
    return logado;
  }

  userId(): string {
    return this.logado() ? this._usuarioLogado.CODIGO_CLIENTE : '';
  }
  
}
