import { Injectable } from '@angular/core';
import { LocalStorageProvider } from '../local-storage/local-storage';
import { UtilsProvider } from '../utils/utils';
import { Cupom } from '../../model/cupom';
import { User } from '../../model/user';

@Injectable({
  providedIn: 'root'
})
export class CarrinhoLocalProvider {

  itens: any[];
  cupom: Cupom;
  cliente: User;
  valor_produto: number = 0;
  desconto: number = 0;
  valor_total: number = 0;  
  quantidadeItens = 0;
  cupom_geral: boolean = true;
  pedido: string;
  forma_pagamento: string;
  codigo_tipo_frete: string;
  teste: string;


  public prazoClubKids(){
		return '3';
	}

	public valorEntregaClubKids($cep){
		// FIXO, aguardando melhoria onde o valor pode variar de acordo com o bairro
		return '0';
	}

	public prazoRetirarLocal(){
		return '3';
	}

	public valorEntregaRetirarLocal(){
		return '0';
  }
  
  constructor(private localStorageProvider: LocalStorageProvider) { 
    console.log("######## CarrinhoLocalProvider ##############")                  
    this.teste = "osman";
    this.itens
  }

  public gravaPedidoCarrinho(pedido) {
    this.pedido = pedido;
    return this.localStorageProvider.set('pedido_carrinho', pedido);
  }

  public gravaFormaPagamentoCarrinho(forma_pagamento) {
    this.forma_pagamento = forma_pagamento;
    return this.localStorageProvider.set('forma_pagamento', forma_pagamento);
  }

  public gravaCodigoTipoFrete(codigo_tipo_frete) {
    this.codigo_tipo_frete = codigo_tipo_frete;
    return this.localStorageProvider.set('codigo_tipo_frete', codigo_tipo_frete);
  }  

  public gravaCupomCarrinho(cupom) {
    this.cupom = cupom;
    this.calculaItens();
    return this.localStorageProvider.set('cupom', cupom);
  }
  
  public gravaCliente(cliente) {
    this.cliente = cliente;
    this.calculaItens();
    return this.localStorageProvider.set('cliente_carrinho', cliente);
  }

  public gravaItens(itens) {
    return this.localStorageProvider.set('itens', itens);
  }

  zeraContadores(){

    return new Promise((resolve, reject) => {
      this.itens = new Array();
      resolve(true);
    });  
  }    
    

  

  calculaItens(){
    this.quantidadeItens = 0;
    this.valor_produto = 0;
    this.valor_total = 0;
    this.desconto = 0;
    
    console.log("Calculado Itens Carrinho");
    let possuiDescontoItem: boolean = false;
    let possuiCupom: boolean = false;
    if (!UtilsProvider.isEmptyObj(this.cupom)){
      possuiCupom = true;
      possuiDescontoItem = this.cupom.PRODUTOS.length > 0;
    }
    
    if (this.itens != null){
      console.log("itens NOT NULLL")
      for (var i = 0; i < this.itens.length; i++) {
        this.quantidadeItens = this.quantidadeItens + this.itens[i].QUANTIDADE;
        this.valor_produto = this.valor_produto + (this.itens[i].QUANTIDADE*this.itens[i].VALOR);
        if (possuiCupom || possuiDescontoItem){
          //Aplica o desconto no Item
          for (var j = 0; j < this.cupom.PRODUTOS.length; j++) {
            if (this.cupom.PRODUTOS[j].CODIGO_PRODUTO == this.itens[i].CODIGO_PRODUTO){
              if (this.cupom.CODIGO_TIPO_DESCONTO =="1"){
                //Desconto em %
                this.desconto = this.desconto + (this.itens[i].QUANTIDADE*this.itens[i].VALOR * this.cupom.VALOR / 100);          
              }else if (this.cupom.CODIGO_TIPO_DESCONTO =="2"){
                //Desconto em Valor
                this.desconto = this.desconto + this.cupom.VALOR;          
              }    
            }
          }
        }
      }
    }else{
      console.log("itens NULLL")
    }
  
    

    if (possuiCupom){
      if (!possuiDescontoItem){
        if (this.cupom.CODIGO_TIPO_DESCONTO =="1"){
          //Desconto em %
          this.desconto = this.valor_produto * this.cupom.VALOR / 100;          
        }else if (this.cupom.CODIGO_TIPO_DESCONTO =="2"){
          //Desconto em Valor
          this.desconto = this.cupom.VALOR;          
        }
      }  
    }

    this.valor_total = this.valor_produto - this.desconto;

  }

  public inicializaCarrinho(){
    var that = this;
    return Promise.resolve()  
    .then(function() {
      console.log("inicializaCarrinho ...");
      return that.zeraContadores();
    })    
    .then(function() {
      console.log("Carrinho zeraContadores OK")            
      return that.getPedidoCarrinho();        
    })     
    .then(function() {
      console.log("Carrinho getPedidoCarrinho OK")            
      return that.getCliente();        
    })     
    .then(function() {
      console.log("Carrinho getCliente OK")            
      return that.getFormaPagamentoCarrinho();        
    })        
    .then(function() {
      console.log("Carrinho getFormaPagamentoCarrinho OK")            
      return that.getCodigoTipoFrete();        
    })        
    .then(function() {
      console.log("Carrinho getCodigoTipoFrete OK")            
      return that.getCupomCarrinho();        
    })      
    .then(function() {
       console.log("Carrinho getCupomCarrinho OK")            
       return that.getItens();        
    })    
    .then(function() {
      console.log("Carrinho getItens OK")            
      return that.calculaItens();        
   })     
    .then(function() {
         console.log("Carrinho calculaItens OK")                    
    });      

  }

  public getItens() {
    return this.localStorageProvider.get('itens')
    .then(
      (itens) => {
        this.itens = itens;
      }
    );
  }

  public getCliente() {
    return this.localStorageProvider.get('cliente_carrinho')
    .then(
      (cliente) =>{
        this.cliente = cliente;
      }
    );
  }

  public getPedidoCarrinho() {
    return this.localStorageProvider.get('pedido_carrinho')
    .then(
      (pedido) =>{
        this.pedido = pedido;
      }
    );
  }
  
  public getFormaPagamentoCarrinho() {
    return this.localStorageProvider.get('forma_pagamento')
    .then(
      (forma_pagamento) =>{
        this.forma_pagamento = forma_pagamento;
      }
    );
  }

  public getCodigoTipoFrete() {
    return this.localStorageProvider.get('codigo_tipo_frete')
    .then(
      (codigo_tipo_frete) =>{
        this.codigo_tipo_frete = codigo_tipo_frete;
      }
    );
  }

  public getCupomCarrinho() {
    return this.localStorageProvider.get('cupom')
    .then(
      (cupom) => {
        this.cupom = cupom;
      }
    );
  }

  public removePedidoCarrinho() {
    this.pedido = null;
    return this.localStorageProvider.remove('pedido_carrinho');
  }

  public removeFormaPagamentoCarrinho() {
    console.log("Removido forma de pagamento");
    this.forma_pagamento = null;
    console.log("forma_pagamento: " + this.forma_pagamento);
    return this.localStorageProvider.remove('forma_pagamento');    
  }

  public removeCodigoTipoFrete() {
    this.codigo_tipo_frete = null;
    return this.localStorageProvider.remove('codigo_tipo_frete');
    
  }

  public removeCupomCarrinho() {
    this.cupom = null;
    this.desconto = 0;
    return this.localStorageProvider.remove('cupom');
  }

  public removeItens() {
    this.itens = [];
    this.quantidadeItens = 0;
    this.valor_produto = 0;
      
    return this.localStorageProvider.remove('itens');     
  }
  
  public removeCliente() {
    this.cliente = null;
    return this.localStorageProvider.remove('cliente_carrinho');     
  }

  public getCarrinhoItensCount() {
    return this.quantidadeItens;    
  }

  public insertItem(itemCarrinho: ItemCarrinho) {

    if (this.itens != null) {
      let index = UtilsProvider.findIndexByKey(this.itens, 'CODIGO_PRODUTO', itemCarrinho.CODIGO_PRODUTO);
      if (index != -1){
        this.itens[index].QUANTIDADE = this.itens[index].QUANTIDADE + itemCarrinho.QUANTIDADE; 
      }else{
        this.itens.push(itemCarrinho);
      }    
    }else{
      this.itens = new Array();
      this.itens.push(itemCarrinho);
    }  
    this.calculaItens();

    return this.gravaItens(this.itens);    
  }

  public updateItem(itemCarrinho: ItemCarrinho) {    
    let index = UtilsProvider.findIndexByKey(this.itens, 'CODIGO_PRODUTO', itemCarrinho.CODIGO_PRODUTO);
    if (index != -1){
      this.itens[index].QUANTIDADE = itemCarrinho.QUANTIDADE;       
    }  
    this.calculaItens();
    return this.gravaItens(this.itens);      
  }

  public removeItem(itemCarrinho: ItemCarrinho) {    
    return new Promise((resolve, reject) => {

      let index = UtilsProvider.findIndexByKey(this.itens, 'CODIGO_PRODUTO', itemCarrinho.CODIGO_PRODUTO);
      
      if (index != -1){
        //delete this.itens[index];
        this.itens.splice(index, 1);      
      }
      
      if (this.itens.length == 0){
        this.limpaCarrinho()
        .then(
          () =>{
            this.calculaItens();
            this.gravaItens(this.itens)
            .then(
              () => {
                resolve(true);
              }
            )
            .catch(
              (erro) => {
                console.error("FALHA GRAVAR CARRINHO. erro: " + erro.message);
                reject(erro);
              }
            )          
          }
        )
        .catch(
          (erro) => {
            console.error("FALHA AO REMOVER ITEM DO CARRINHO. erro: " + erro.message);
            reject(erro);
          }
        )
      }else{
        this.calculaItens();
        this.gravaItens(this.itens)
        .then(
          () => {
            resolve(true);
          }
        )
        .catch(
          (erro) => {
            console.error("FALHA GRAVAR CARRINHO. erro: " + erro.message);
            reject(erro);
          }
        )      
      }    
    }); 
  }
  
  public limpaCarrinho() {
    var that = this;
    return Promise.resolve()
      .then(function() {
        console.log("removeCupomCarrinho");
        return that.removeCupomCarrinho();
      }) 
      .then(function() {
        return that.removePedidoCarrinho();
      }) 
      .then(function() {
        return that.removeFormaPagamentoCarrinho();
      }) 
      .then(function() {
        return that.removeCodigoTipoFrete();
      }) 
      .then(function() {
        return that.removeCliente();
      }) 
      .then(function() {
        console.log("remove Itens");
        return that.removeItens();
      });    
  }
    

}

export class ItemCarrinho {
  ID: number;
  CODIGO_PRODUTO: number;
  VALOR: number;
  DESCONTO: number;
  QUANTIDADE: number;
  TITULO: string;
  FOTO: string;
  QUANTIDADE_DISPONIVEL: number;
}
