import { Injectable } from '@angular/core';

import { Network } from '@ionic-native/network/ngx';

import { BehaviorSubject, Observable } from 'rxjs';


 export enum ConnectionStatus {
     Online,
     Offline
 }

@Injectable({
    providedIn: 'root'
  })
export class NetworkProvider {

    public status: ConnectionStatus;
    private _status: BehaviorSubject<ConnectionStatus> = new BehaviorSubject(null);

    constructor(
        public network: Network
    ) {
        if (this.network.type =='none'){
          this.setStatus(ConnectionStatus.Offline)
        }else{
          this.setStatus(ConnectionStatus.Online)
        }        
    }

    public initializeNetworkEvents(): void {

        /* OFFLINE */
        this.network.onDisconnect().subscribe(() => {
            if (this.status === ConnectionStatus.Online) {
                this.setStatus(ConnectionStatus.Offline);
            }
        })

        /* ONLINE */
        this.network.onConnect().subscribe(() => {
            if (this.status === ConnectionStatus.Offline) {
                this.setStatus(ConnectionStatus.Online);
            }
        })

    }

    public getNetworkType(): string {
        return this.network.type
    }

    public getNetworkStatus(): Observable<ConnectionStatus> {
        return this._status.asObservable();
    }
    
    private setStatus(status: ConnectionStatus) {
        this.status = status;
        this._status.next(this.status);
    }
}